# mypy: disallow_untyped_defs=False
# copykat.py
# Written by G.D. Wallters
# Freely released the code to recipe_scraper group
# 8 February, 2020
# =======================================================


from ._abstract import AbstractScraper
from ._utils import normalize_string


class CopyKat(AbstractScraper):
    @classmethod
    def host(cls):
        return "copykat.com"

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()

    def description(self):
        d = normalize_string(self.soup.find("span", {"style": "display: block;"}).text)

        return d if d else None



from recipe_scrapers.foodandwine import FoodAndWine
from tests import ScraperTest


class TestFoodAndWineScraper(ScraperTest):

    scraper_class = FoodAndWine

    def test_host(self):
        self.assertEqual("foodandwine.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.foodandwine.com/recipes/kwames-pepper-shrimp",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual("Kwame's Pepper Shrimp", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Kwame Onwuachi")

    def test_image(self):
        self.assertEqual(
            "https://www.foodandwine.com/thmb/BZ8hhoqmF0B8KlP_pHSEsxjhxnQ=/1500x0/filters:no_upscale():max_bytes(150000):strip_icc()/pepper-shrimp-FT-RECIPE0920-9b30018088cc419f88c91fa33564075c.jpg",
            self.harvester_class.image(),
        )

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1/2 cup unsalted butter (4 ounces)",
                "5 large garlic cloves, minced",
                "1 fresh Scotch bonnet chile, stemmed, unseeded, and minced (about 1 tablespoon)",
                "1 1/2 teaspoons minced peeled fresh ginger",
                "1/2 cup fresh lime juice",
                "1/2 cup distilled white vinegar",
                "1 thyme sprig",
                "3 pounds unpeeled head-on raw large shrimp, deveined",
                "1 teaspoon kosher salt",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Melt butter in a large pot over medium. Add garlic, chile, and ginger; cook, stirring often, until aromatic and tender, about 5 minutes. Add lime juice, vinegar, and thyme; bring to a simmer over medium. Increase heat to medium-high; add shrimp, and cook, stirring often, until shrimp are opaque and cooked through, about 5 minutes. Remove from heat. Sprinkle with salt, and serve immediately.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import normalize_string


class LekkerEnSimpel(AbstractScraper):
    @classmethod
    def host(cls):
        return "lekkerensimpel.com"

    def author(self):
        return self.schema.author()

    def title(self):
        title = self.soup.find("h1", {"class": "hero__title"}).text
        return normalize_string(title)

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        image = self.soup.find("meta", {"property", "og:image"})
        return image["content"] if image else None

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.schema.description()

    def language(self):
        return "nl-NL"



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class USDAMyPlate(AbstractScraper):
    @classmethod
    def host(cls):
        return "myplate.gov"

    def title(self):
        return self.soup.h1.get_text().strip()

    def total_time(self):
        # not in every recipe has time given
        full_detail = self.soup.find(
            "div", {"class": "mp-recipe-full__overview desktop:grid-col-5 grid-row"}
        )

        minutes = 0
        for span in full_detail.findAll(
            "span", {"class": "mp-recipe-full__detail--data"}
        ):
            if "minute" in span.get_text().lower() or "hour" in span.get_text().lower():
                minutes += get_minutes(span)

        if minutes == 0:
            return None

        return minutes

    def yields(self):
        full_detail = self.soup.find(
            "div", {"class": "mp-recipe-full__overview desktop:grid-col-5 grid-row"}
        )

        spans = full_detail.findAll("span")
        i = 0
        for span in spans:
            if "Makes:" in span:
                return get_yields(spans[i + 1])
            i += 1

    def image(self):
        div = self.soup.find(
            "div",
            {
                "class": "field field--name-field-recipe-image field--type-image field--label-visually_hidden"
            },
        )
        url = div.find("img")["src"]
        # return only the portion before the question mark
        return url.split("?")[0]

    def ingredients(self):
        ingredients = self.soup.find(
            "div", {"class": "field--name-field-ingredients"}
        ).findAll("li")

        return [normalize_string(paragraph.get_text()) for paragraph in ingredients]

    def instructions(self):
        div = self.soup.find(
            "div",
            {
                "class": "clearfix text-formatted field field--name-field-instructions field--type-text-long field--label-above"
            },
        )
        instructions = div.find("div", {"class", "field__item"})

        return "\n".join(instructions.stripped_strings)

    def nutrients(self):
        nutrition = {}

        table = self.soup.find(
            "form", {"class": "mp-recipe-full__nutrition-form"}
        ).find("table")
        rows = table.find_all("tr")

        elements = []
        for row in rows:
            cols = row.find_all("td")
            cols = [ele.text.strip() for ele in cols]
            elements.append([ele for ele in cols if ele])

        for el in elements:
            if len(el) > 1:
                nutrition[el[0]] = el[1]

        return nutrition

    def serving_size(self):
        return normalize_string(
            self.soup.find("div", {"class": "field--name-field-recipe-serving-size"})
            .find("span", {"class": "field__item"})
            .get_text()
        )

    def description(self):
        return normalize_string(
            self.soup.find("div", {"class": "mp-recipe-full__description"})
            .find("p")
            .get_text()
        )

    def recipe_source(self):
        return normalize_string(
            self.soup.find("span", {"class": "field--name-field-source"})
            .find("p")
            .get_text()
        )



from recipe_scrapers.thehappyfoodie import TheHappyFoodie
from tests import ScraperTest


class TestTheHappyFoodie(ScraperTest):

    scraper_class = TheHappyFoodie
    test_file_name = "thehappyfoodie_2"

    def test_host(self):
        self.assertEqual("thehappyfoodie.co.uk", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "The Ottolenghi Test Kitchen’s M.E. Mac ‘n’ Cheese with Za’atar Pesto",
        )

    def test_total_time(self):
        self.assertEqual(55, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "300g dried cavatappi or fusilli pasta",
                "600–700ml whole milk",
                "65g unsalted butter, cut into roughly 3cm cubes",
                "3 garlic cloves, crushed",
                "⅛ tsp ground turmeric",
                "1½ tsp cumin seeds, toasted and roughly crushed with a pestle and mortar",
                "75ml double cream",
                "150g mature cheddar, roughly grated",
                "180g Greek feta, roughly crumbled",
                "salt and black pepper",
                "45g crispy onions or shallots, store-bought or homemade (see intro), to serve",
                "1 large lemon",
                "3 tbsp za’atar",
                "20g fresh coriander, roughly chopped",
                "1 garlic clove, roughly chopped",
                "40g pine nuts, lightly toasted",
                "90ml olive oil",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            [
                "Put the pasta, 600ml of milk, 350ml of water, the butter, garlic, turmeric, 1 teaspoon of salt and a good grind of pepper into a large sauté pan on a medium-high heat. Bring to a simmer, then turn the heat down to medium and cook, stirring occasionally, for 10–14 minutes, or until the pasta is al dente and the sauce has thickened from the pasta starches (it will still be quite saucy). If using cavatappi, you might need to add the extra 100ml of milk at this stage depending on how saucy you like your mac ’n’ cheese. Turn the heat down to low and stir through the cumin, cream and both cheeses until the cheddar is nicely melted.",
                "While the pasta is cooking, make the pesto. Finely grate the lemon to give you 1 teaspoon of zest. Then use a small, sharp knife to peel and segment the lemon and roughly chop the segments. Place in a bowl with the lemon zest and set aside. Put the za’atar, coriander, garlic, pine nuts, ⅛ teaspoon of salt, a good grind of pepper and half the oil into a food processor and pulse a few times until you have a coarse paste. Add to the chopped lemon in the bowl and stir in the remaining oil.",
                "Transfer the mac ’n’ cheese to a large serving platter with a lip or a shallow bowl, dot all over with the pesto, then top with the crispy onions.",
                "Get ahead: Make the mac ’n’ cheese ahead of time if you like, adding a splash of water to thin out when reheating.",
                "Make it your own:",
                "– Play with different cheeses and spices.",
                "– Use different pasta shapes like macaroni, adjusting liquid levels as necessary.",
            ],
            self.harvester_class.instructions().splitlines(),
        )

    def test_author(self):
        return self.assertEqual("Yotam Ottolenghi", self.harvester_class.author())

    def test_image(self):
        return self.assertEqual(
            "https://thehappyfoodie.co.uk/wp-content/uploads/2022/02/Ottolenghi-Middle-Eastern-Mac-and-Cheese-scaled.jpg",
            self.harvester_class.image(),
        )

    def test_cuisine(self):
        return self.assertEqual("Middle Eastern", self.harvester_class.cuisine())

    def test_category(self):
        return self.assertEqual("Main Course", self.harvester_class.category())



from recipe_scrapers.woop import Woop
from tests import ScraperTest


class TestWoopScraper(ScraperTest):
    scraper_class = Woop

    def test_host(self):
        self.assertEqual("woop.co.nz", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Pan-seared beef",
        )

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 pack of beef sirloin",
                "1 pack of steamed brown rice",
                "1 pot of Korean seasoning",
                "1 pot of sesame drizzle",
                "1 pot of kimchi",
                "1 sachet of sesame sprinkle",
                "1 pot of edamame and peas",
                "1 capsicum",
                "1 bag of bok choy",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            """1. TO PREPARE THE VEGETABLES: Slice the capsicum into ½ cm slices. Cut the root end off the bok choy, discard and slice into 1 cm slices. Finely slice the kimchi.
2. TO COOK THE KIMCHI RICE: Spray a non-stick frying pan with oil and place over a medium-high heat. Once hot add the sliced capsicum and cook for 2-3 mins stirring occasionally to create a char. Open the bag of steamed brown rice, squeezing gently on the bag to break up any large clumps and add to the pan with the bok choy, edamame and peas and cook for 1-2 mins. Pour in the Korean seasoning and add half the kimchi, stir and cook for 1-2 mins. Season with salt to taste, remove from the pan and cover to keep warm.
3. TO COOK THE BEEF SIRLOIN: Remove the beef sirloin from its packaging and pat dry with a paper towel. Respray the pan with oil and place over a high heat. Season the beef with salt and pepper and place in the hot pan. Cook for 2-3 mins each side for medium-rare, a little longer for well done. Rest for 1-2 mins before slicing.
TO SERVE: Spoon kimchi fried rice onto plates and top with sliced beef sirloin. Dollop with sesame drizzle and sprinkle remaining kimchi and sesame sprinkle on top.""",
            self.harvester_class.instructions(),
        )

    def test_nutrients(self):
        return self.assertEqual(
            {
                "Energy": "2322kj (554Kcal)",
                "Protein": "43g",
                "Carbohydrate": "43g",
                "Fat": "23g",
                "Contains": "Soy, Sesame, Gluten, Milk, Egg",
            },
            self.harvester_class.nutrients(),
        )

    def test_image(self):
        self.assertEqual(
            "https://woop.co.nz/media/catalog/product/cache/f4f005ad5960ef8c7b8a08a9a3fc244e/b/-/b-pan-seared-beef_s64dydtpoves1rbu.jpg",
            self.harvester_class.image(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class SimplyRecipes(AbstractScraper):
    @classmethod
    def host(cls):
        return "simplyrecipes.com"

    def title(self):
        return self.soup.find("h1").get_text()

    def total_time(self):
        return get_minutes(
            self.soup.find("div", {"class": "total-time"})
            .find("span", {"class": "meta-text__data"})
            .text
        )

    def yields(self):
        return get_yields(
            normalize_string(
                self.soup.find("div", {"class": "recipe-serving"})
                .find("span", {"class": "meta-text__data"})
                .text
            )
        )

    def ingredients(self):
        ingredients = self.soup.find(
            "ul", {"class": "structured-ingredients__list"}
        ).findAll("li")

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        steps = self.soup.find(
            "div", {"class": "structured-project__steps"}
        ).ol.findAll("li")

        return "\n".join(
            [
                normalize_string(
                    step.div.text + "".join([p.text for p in step.findAll("p")])
                )
                for step in steps
            ]
        )



from recipe_scrapers.chefkoch import Chefkoch
from tests import ScraperTest


class TestChefkochScraper(ScraperTest):

    scraper_class = Chefkoch

    def test_host(self):
        self.assertEqual("chefkoch.de", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.chefkoch.de/rezepte/1170311223132029/Hackbraten-supersaftig.html",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Hackbraten supersaftig")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Delphinella")

    def test_description(self):
        self.assertEqual(
            self.harvester_class.description(),
            "Hackbraten supersaftig - saftiger Hackbraten mit viel Soße. Über 1290 Bewertungen und für lecker befunden. Mit ► Portionsrechner ► Kochbuch ► Video-Tipps!",
        )

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://img.chefkoch-cdn.de/rezepte/1170311223132029/bilder/1435439/crop-960x540/hackbraten-supersaftig.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 ½ Semmel(n) , altbacken",
                "2 Gewürzgurke(n)",
                "2 kleine Zwiebel(n)",
                "1 kl. Bund Petersilie",
                "2 EL Zitronensaft",
                "50 g Butter",
                "600 g Hackfleisch, gemischt",
                "2 kleine Ei(er)",
                "125 ml Fleischbrühe",
                "125 ml Sahne",
                "1 EL Crème fraîche",
                "1 TL Paprikapulver, edelsüß",
                "Salz und Pfeffer, schwarzer",
                "Cayennepfeffer",
                "Fett für die Form",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Die Semmeln in Scheiben schneiden und mit Wasser übergießen, quellen lassen. Gut ausdrücken. Die Gewürzgurken in sehr feine Würfel schneiden. Zwiebeln ebenfalls in feine Würfel schneiden.\n\n1 EL Butter erhitzen und die Zwiebeln glasig anschwitzen. Petersilie dazugeben. \n\nZwiebel-Petersilienmischung in eine Schüssel geben. Semmeln, Gewürzgurken, Hackfleisch, Eier und Zitronensaft zufügen. Alles mit Salz, Cayennepfeffer und schwarzem Pfeffer würzen und kräftig durchkneten. \n\nDie restliche Butter schmelzen, eine Form fetten. Den Fleischteig zu einem Laib formen und in die Form legen. Auf der unteren Schiene 30 Minuten (Umluft 180 °C) backen, dabei immer mit der flüssigen Butter bestreichen. \n\nDie Fleischbrühe erhitzen und mit der Sahne, der Crème fraîche und dem Paprikapulver verrühren. (wer sehr viel Soße mag, kann die Soßenmenge einfach verdoppeln). Die Soße über den Hackbraten gießen und weitere 10 - 15 Minuten garen. \n\nDazu passen hervorragend Salzkartoffeln.",
            self.harvester_class.instructions(),
        )

    def test_total_time(self):
        return self.assertEqual(85.0, self.harvester_class.total_time())

    def test_cook_time(self):
        return self.assertEqual(45.0, self.harvester_class.cook_time())

    def test_prep_time(self):
        return self.assertEqual(40.0, self.harvester_class.prep_time())



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper
from ._utils import get_yields, normalize_string


class IG(AbstractScraper):
    @classmethod
    def host(cls):
        return "receitas.ig.com.br"

    def title(self):
        return self.soup.find("h2", {"itemprop": "name"}).get_text()

    def author(self):
        nav = self.soup.find("nav", {"class": "nav-mais-receitas"})
        if nav:
            first = nav.find("li", {"class": "first"})
            if first:
                return first.find("a").get_text()

    def total_time(self):
        container = self.soup.find("div", {"class": "box-info-preparacao"})
        if not container:
            return None
        else:
            return int(container.find("span", {"class": "valor"}).get_text())

    def yields(self):
        container = self.soup.find("div", {"class": "box-info-rendimento"})
        if not container:
            return None
        else:
            return get_yields(container.find("span", {"class": "valor"}).get_text())

    def image(self):
        container = self.soup.find("div", {"class": "box-img-receita"})
        if not container:
            return None

        image = container.find("img", {"src": True})
        return image["src"] if image else None

    def ingredients(self):
        ingredients = self.soup.find("ul", {"class", "lista-ingredientes"}).findAll(
            "li"
        )

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = self.soup.find("div", {"class": "box-preparo"})
        return normalize_string(instructions.get_text())



from recipe_scrapers.kingarthur import KingArthur
from tests import ScraperTest


class TestKingArthurScraper(ScraperTest):

    scraper_class = KingArthur

    def test_host(self):
        self.assertEqual("kingarthurbaking.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.kingarthurbaking.com/recipes/spiced-rye-ginger-cookies-recipe",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Spiced Rye Ginger Cookies")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "King Arthur Flour")

    def test_yields(self):
        self.assertEqual("22 items", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.kingarthurbaking.com/sites/default/files/2021-11/Spiced-Rye-Ginger_SOCIAL_HERO_33683-1.jpg",
            self.harvester_class.image(),
        )

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_rating(self):
        self.assertEqual(4.80, self.harvester_class.ratings())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 cups (212g) medium rye flour",
                "1 teaspoon baking soda",
                "1/2 teaspoon salt",
                "2 teaspoons ginger",
                "3/4 teaspoon black pepper",
                "1/2 teaspoon cinnamon",
                "1/2 teaspoon cardamom",
                "3/4 cup (149g) granulated sugar",
                "1/2 cup (99g) vegetable oil",
                "1 large egg",
                "1/4 cup (85g) molasses",
                "1/3 cup (76g) coarse sparkling sugar or 1/3 cup (66g) granulated sugar",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            """Preheat the oven to 325°F. Lightly grease (or line with parchment) two baking sheets.\nWeigh your flour; or measure it by gently spooning it into a cup, then sweeping off any excess.\nIn a medium bowl, whisk together the flour, baking soda, salt, and spices. Set aside.\nIn a large mixing bowl, using either a hand whisk, an electric mixer, or a stand mixer, whisk the sugar and oil until combined.\nAdd the egg and whisk until smooth.\nStir in the molasses.\nAdd the dry ingredients to the bowl and stir until well combined.\nUse a spoon (or a tablespoon cookie scoop) to portion 1 1/4" balls of dough.\nRoll the dough balls in granulated or sparkling sugar to coat before placing onto the prepared baking sheets. Leave 2" between them on all sides; they'll spread as they bake.\nBake the cookies for 12 to 15 minutes, until they're puffed and their edges are set.\nRemove the cookies from the oven, and cool completely right on the pan.\nStore cookies, well wrapped, at room temperature for several days; freeze for longer storage.""",
            self.harvester_class.instructions(),
        )


class TestKingArthurScraperBeautifulBuns(ScraperTest):

    scraper_class = KingArthur
    test_file_name = "kingarthur_beautiful_buns"

    def test_host(self):
        self.assertEqual("kingarthurbaking.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.kingarthurbaking.com/recipes/beautiful-burger-buns-recipe",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Beautiful Burger Buns")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "King Arthur Flour")

    def test_yields(self):
        self.assertEqual("8 items", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.kingarthurbaking.com/sites/default/files/2021-08/beautiful-burger-buns.jpg",
            self.harvester_class.image(),
        )

    def test_total_time(self):
        self.assertEqual(160, self.harvester_class.total_time())

    def test_rating(self):
        self.assertEqual(4.70, self.harvester_class.ratings())

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 1/2 cups (420g) King Arthur Unbleached All-Purpose Flour",
                "3/4 to 1 cup (170g to 227g) water lukewarm*",
                "2 tablespoons (28g) butter at room temperature",
                "1 large egg",
                "1/4 cup (50g) granulated sugar",
                "1 1/4 teaspoons salt",
                "1 tablespoon (9g) instant yeast",
                "3 tablespoons (43g) butter melted; divided",
                "1 large egg white whisked with 2 tablespoons cold water*",
                "sesame seeds or the seeds of your choice",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            """Weigh your flour; or measure it by gently spooning it into a cup, then sweeping off any excess.\nTo make the dough: Mix and knead all of the dough ingredients — by hand, mixer, or bread machine — to make a soft, smooth dough.\nCover the dough and let it rise until it's nearly doubled in bulk, about 1 to 2 hours.\nTo shape the buns: Gently deflate the dough and divide it into eight pieces (about 100g each); to make smaller or larger buns see "tips," below. Shape each piece into a ball.\nFlatten each dough ball with the palm of your hand until it's about 3" across.\nPlace the buns on a lightly greased or parchment-lined baking sheet. Cover and let rise until noticeably puffy, about an hour. Toward the end of the rising time, preheat the oven to 375°F.\nBrush the buns with about half of the melted butter. To make seeded buns, brush the egg white/water mixture right over the melted butter; it'll make the seeds adhere. Sprinkle buns with the seeds of your choice.\nTo bake the buns: Bake the buns for 15 to 18 minutes, until golden. Remove them from the oven and brush with the remaining melted butter; this will give the buns a satiny, buttery crust. If you've made seeded buns apply the melted butter carefully, to avoid brushing the seeds off the buns.\nCool the buns on a rack before slicing in half, horizontally. Use as a base for burgers (beef or plant-based) or any favorite sandwich filling.\nStorage information: Store leftover buns, well-wrapped, at room temperature for several days; freeze for longer storage.""",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.davidlebovitz import DavidLebovitz
from tests import ScraperTest


class TestDavidLebovivtzScraper(ScraperTest):

    scraper_class = DavidLebovitz

    def test_host(self):
        self.assertEqual("davidlebovitz.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("David", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Faux Gras", self.harvester_class.title())

    def test_image(self):
        self.assertEqual(
            "https://www.davidlebovitz.com/wp-content/uploads/2015/06/Faux-Gras-Lentil-Pate-8.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "12 medium-sized (100g, about 1 cup) button mushrooms",
                "2 tablespoons olive oil",
                "2 tablespoons butter (salted or unsalted)",
                "1 small onion (peeled and diced)",
                "2 cloves garlic (peeled and minced)",
                "2 cups (400g) cooked green lentils",
                "1 cup (140g) toasted walnuts or pecans",
                "2 tablespoons freshly squeezed lemon juice",
                "1 tablespoon soy sauce or tamari",
                "2 teaspoons minced fresh rosemary",
                "2 teaspoons fresh thyme (minced)",
                "2 tablespoons fresh sage or flat leaf parsley",
                "optional: 2 teaspoons Cognac or brandy",
                "1 teaspoon brown sugar",
                "1/8 teaspoon cayenne pepper",
                "salt and freshly ground black pepper",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Wipe the mushrooms clean. Slice off a bit of the stem end (the funky parts) and slice them. Heat the olive oil and butter in a skillet or wide saucepan. Add the onions and garlic, and cook, stirring frequently, until the onions become translucent, 5 to 6 minutes. Add the mushrooms and cook, stirring occasionally, until they’re soft and cooked through, another 5 to 8 minutes. Remove from heat.\nIn a food processor, combine the cooked lentils, nuts, lemon juice, soy sauce, rosemary, thyme, sage or parsley, Cognac (if using), brown sugar, and cayenne. Scrape in the cooked mushroom mixture and process until completely smooth. Taste, and add salt, pepper, and additional cognac, soy sauce, or lemon juice, if it needs balancing.\nScrape the pâté into a small serving bowl and refrigerate for a few hours, until firm.""",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            """Adapted from Très Green, Très Clean, Très Chic by Rebecca Leffler Lentils double in volume when cooked, so 1 cup (160g) of dried lentils will yield close to the correct amount. They usually take about 20 to 30 minutes to cook until soft, but check the directions on the package for specific guidelines. If avoiding gluten, use tamari instead of soy sauce. For a vegan version, replace the butter with the same quantity of olive oil, for a total of 1/4 cup (60ml) of olive oil. The cognac or brandy is optional, but it does give the faux gras a little je ne sais quoi.""",
            self.harvester_class.description(),
        )



import unittest

from recipe_scrapers.plugins import HTMLTagStripperPlugin


class TestHTMLTagStripperPlugin(unittest.TestCase):
    def sample_ingredients_method(self, *args, **kwargs):
        return [
            "3 tbsp. <p>extra-virgin olive oil, divided</p>",
            "<p>Juice of 1 lemon</p>",
            "3 <p>cloves garlic, minced</p>",
        ]

    def sample_title_method(self, *args, **kwargs):
        return "Sticky Pomegranate &amp;amp; Black Pepper Chicken Wing"

    def sample_instructions_method(self, *args, **kwargs):
        return """
            blah blah \n\n
            blah <p>should remove ps</p>
        """

    def test_sample_ingredients_method(self):
        # the default method is intact
        self.assertEqual(
            self.sample_ingredients_method(),
            [
                "3 tbsp. <p>extra-virgin olive oil, divided</p>",
                "<p>Juice of 1 lemon</p>",
                "3 <p>cloves garlic, minced</p>",
            ],
            "Original method should be as defined in the test class",
        )

        # we "decorate" our  sample_instructions_method as we would in _abstract.py with the
        # HTMLTagStripper plugin if it is used. Note how the result returned from the method
        # is changed as the plugin kicks in
        name = "sample_ingredients_method"
        setattr(
            self.__class__,
            name,
            HTMLTagStripperPlugin.run(getattr(self.__class__, name)),
        )

        return self.assertEqual(
            self.sample_ingredients_method(),
            [
                "3 tbsp. extra-virgin olive oil, divided",
                "Juice of 1 lemon",
                "3 cloves garlic, minced",
            ],
            "Result must have html tags stripped when invoked after plugin used",
        )

    def test_sample_instructions_method(self):
        # the default method is intact
        self.assertEqual(
            self.sample_instructions_method(),
            """
            blah blah \n\n
            blah <p>should remove ps</p>
        """,
            "Original method should be as defined in the test class",
        )

        # we "decorate" our  sample_instructions_method as we would in _abstract.py with the
        # HTMLTagStripper plugin if it is used. Note how the result returned from the method
        # is changed as the plugin kicks in
        name = "sample_instructions_method"
        setattr(
            self.__class__,
            name,
            HTMLTagStripperPlugin.run(getattr(self.__class__, name)),
        )

        return self.assertEqual(
            self.sample_instructions_method(),
            """
            blah blah \n\n
            blah should remove ps
        """,
            "Result must have html tags stripped when invoked after plugin used",
        )

    def test_sample_title_method(self):
        self.assertEqual(
            self.sample_title_method(),
            "Sticky Pomegranate &amp;amp; Black Pepper Chicken Wing",
            "Original method should be as defined in the test class",
        )

        # we "decorate" our  sample_instructions_method as we would in _abstract.py with the
        # HTMLTagStripper plugin if it is used. Note how the result returned from the method
        # is changed as the plugin kicks in
        name = "sample_title_method"
        setattr(
            self.__class__,
            name,
            HTMLTagStripperPlugin.run(getattr(self.__class__, name)),
        )

        return self.assertEqual(
            self.sample_title_method(),
            "Sticky Pomegranate & Black Pepper Chicken Wing",
            "Result must have html tags stripped when invoked after plugin used",
        )



from recipe_scrapers.practicalselfreliance import PracticalSelfReliance
from tests import ScraperTest


class TestPracticalSelfRelianceScraper(ScraperTest):

    scraper_class = PracticalSelfReliance

    def test_host(self):
        self.assertEqual("practicalselfreliance.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://creativecanning.com/zucchini-relish/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Zucchini Relish Recipe for Canning"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Ashley Adamant")

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_total_time(self):
        self.assertEqual(150, self.harvester_class.total_time())

    def test_image(self):
        self.assertEqual(
            "https://creativecanning.com/wp-content/uploads/2021/02/Zucchini-Relish-61-720x720.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 cups zucchini, diced (about 3 medium)",
                "1 cup onion, diced (about 1 medium)",
                "1 cup red bell pepper, diced (about 2 small or 1 large)",
                "2 Tablespoons Salt (pickling and canning salt, or kosher salt)",
                "1 3/4 cups sugar",
                "2 teaspoons celery seed (whole)",
                "1 teaspoon mustard seed (whole)",
                "1 cup cider vinegar (5% acidity)",
                "Pickle Crisp Granules (optional, helps veggies stay firm after canning)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Wash vegetables.\nRemove stem and blossom ends from zucchini and dice into 1/4 to 1/2 inch pieces. Measure 2 cups.\nPeel and dice onion. Measure 1 cup.\nPeel and dice onion. Measure 1 cup.\nStem and seed peppers, then dice. Measure 1...\nStem and seed peppers, then dice. Measure 1 cup.\nImportant, don't skip this step! Combine diced vegetables in a large bowl and sprinkle salt over the top. Stir gently to distribute the salt, then add water until vegetables are completely submerged. Allow the vegetables to soak in the saltwater for 2 hours, then drain completely.\nPrepare a water bath canner (optional, only if canning).\nIn a separate saucepan or stockpot, bring vinegar, sugar, and spices to a gentle simmer (180 degrees F). Do not add salt, the salt is only used to soak veggies before draining.\nAdd drained vegetables to the simmering vinegar/spices and gently simmer for 10 minutes.\nPack hot relish into prepared half-pint or pint jars, leaving 1/2 inch headspace.\nIf not canning, just seal jars and allow them to cool on the counter before storing in the refrigerator.\nIf canning, de-bubble jars, wipe rims, and adjust headspace to ensure 1/2 inch. Seal with 2 part canning lids.\nProcess in a water bath canner for 10 minutes, then turn off the heat. Allow the jars to sit in the canner for another 5 minutes to cool slightly, then remove the jars to cool on a towel on the counter.\nLeave the jars undisturbed for 24 hours, then check seals. Store any unsealed jars in the refrigerator for immediate use. Properly canned and sealed jars should maintain peak quality on the pantry shelf for 12-18 months.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
from typing import Dict, Optional, Tuple, Union

from requests import Session

from ._abstract import HEADERS, AbstractScraper


class BettyBossi(AbstractScraper):
    """Scrape BettyBossi.ch recipes.

    This scraper is particular as the website implements a refresh after
    loading the page the first time. It is therefore needed to do two get
    requests in a single session, once to initialize the connection, the second
    to load the page content.
    """

    @classmethod
    def host(cls):
        return "bettybossi.ch"

    def __init__(
        self,
        url: str,
        proxies: Optional[
            Dict[str, str]
        ] = None,  # allows us to specify optional proxy server
        timeout: Optional[
            Union[float, Tuple[float, float], Tuple[float, None]]
        ] = None,  # allows us to specify optional timeout for request
        wild_mode: Optional[bool] = False,
        html: Union[str, bytes, None] = None,
    ) -> None:
        if html is None:
            with Session() as session:
                session.proxies.update(proxies or {})
                session.headers.update(HEADERS)

                session.get(url, timeout=timeout)
                html = session.get(url, timeout=timeout).content  # reload the page

        # As the html content is provided, the parent will not query the page
        super().__init__(url, proxies, timeout, wild_mode, html)

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.schema.description()



from recipe_scrapers.budgetbytes import BudgetBytes
from tests import ScraperTest


class TestBudgetBytesScraper(ScraperTest):

    scraper_class = BudgetBytes

    def test_host(self):
        self.assertEqual("budgetbytes.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.budgetbytes.com/creamy-coconut-curry-lentils-with-spinach/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Creamy Coconut Curry Lentils with Spinach"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Beth - Budget Bytes")

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 items", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 Tbsp olive oil ($0.24)",
                "2 cloves garlic ($0.16)",
                "1 tsp grated fresh ginger ($0.10)",
                "1 small yellow onion ($0.21)",
                "1 Tbsp curry powder* ($0.30)",
                "1 cup brown lentils (dry) ($0.67)",
                "2 cups vegetable broth** ($0.26)",
                "1 13oz. can coconut milk ($1.99)",
                "3 cups fresh baby spinach ($1.61)",
                "4 cups cooked rice ($0.60)",
                "1/4 cup chopped fresh cilantro ($0.15)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Mince the garlic, grate the ginger, and dice the onion. Add the olive oil, garlic, and ginger to a deep skillet, Dutch oven, or soup pot. Sauté the garlic and ginger over medium heat for 1 minute, or just until the garlic becomes soft and fragrant.\nAdd the diced onion to the skillet and continue to sauté over medium until the onion is soft and translucent. Add the curry powder and continue to sauté for about one minute more to toast the spices.\nAdd the dry lentils and vegetable broth to the skillet. Stir to dissolve any browned bits from the bottom of the skillet. Place a lid on top, turn the heat up to medium-high, and bring the broth to a boil. Once boiling, turn the heat down to low, and let it simmer for 20 minutes, stirring occasionally.\nAfter simmering for 20 minutes the lentils should be tender and most of the broth absorbed. Add the can of coconut milk and stir to combine. Turn the heat back up to medium and allow the skillet to come back up to a simmer. Let it simmer without a lid for an additional 10 minutes, stirring often, to thicken the mixture.\nOnce thickened, turn the heat off. Add the fresh spinach and stir gently until the spinach has wilted. Taste the mixture and adjust the salt or curry powder to your liking, if needed.\nServe over a bowl of rice, and top with chopped cilantro if desired.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper


class OneHundredOneCookBooks(AbstractScraper):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.soup = self.soup.find("div", id="recipe")

    @classmethod
    def host(cls):
        return "101cookbooks.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.soup.find("h1").get_text()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        data = self.soup.find_all("p", limit=3, recursive=False)[-1].get_text()
        extraction = re.search("([0-9]+) servings", data)
        return extraction.group(1) if extraction else None

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ingredients = self.soup.find("blockquote").p.stripped_strings
        return list(ingredients)

    def instructions(self):
        return self.soup.find_all("p", limit=2, recursive=False)[1].get_text(
            "\n", strip=True
        )

    def ratings(self):
        return None



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields


class Meljoulwan(AbstractScraper):
    @classmethod
    def host(cls):
        return "meljoulwan.com"

    def author(self):
        return (
            self.soup.find("div", {"class": "post-author"})
            .findChild("span")
            .get_text()
            .strip()
        )

    def title(self):
        return (
            self.soup.find("div", {"class": "recipe-post"})
            .findChild("h2")
            .get_text()
            .strip()
        )

    def category(self):
        ul_list = (
            self.soup.find("div", {"class": "post-info"})
            .findChild("div", {"class", "post-category"})
            .findChildren("a")
        )

        categories = []
        for li in ul_list:
            if li.get_text() != "Blog":
                categories.append(li.get_text())
        return ",".join(categories)

    def total_time(self):
        infostring = (
            self.soup.find("div", {"class": "recipe-copy"}).find("em").get_text()
        )

        matches = re.search(
            r"(Cook|Total time:)\s(\d+\-?\d+)\s\bmin(utes)?", infostring
        )

        return get_minutes(matches.group(2))

    def yields(self):
        infostring = (
            self.soup.find("div", {"class": "recipe-copy"})
            .find("em")
            .get_text()
            .strip()
        )

        matches = re.search(r"^Serves\s(\d+\-?\–?\d+)", infostring)

        return get_yields(matches.group(1))

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ul_list = self.soup.find("div", {"class": "tabbed-list"}).findChildren("ul")

        ingredients = []
        for ul in ul_list:
            li_list = ul.findChildren("li")
            for li in li_list:
                ingredients.append(li.get_text().strip())

        return ingredients

    def instructions(self):
        ul_list = self.soup.find("div", {"class": "numbered-list"}).findChildren(
            "div", {"class", "num-list-group"}
        )

        count = 0
        instructions = []
        for li in ul_list:
            count += 1
            instructions.append(
                str(count)
                + ". "
                + li.findChild("div", {"class": "num-list-copy"}).get_text().strip()
            )
        return "\n".join(instructions)



from recipe_scrapers.purplecarrot import PurpleCarrot
from tests import ScraperTest


class TestPurpleCarrotScraper(ScraperTest):

    scraper_class = PurpleCarrot

    def test_host(self):
        self.assertEqual("purplecarrot.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.purplecarrot.com/recipe/roasted-cauliflower-lentil-bowl-with-avocado-curried-balsamic-vinaigrette",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            "Roasted Cauliflower Lentil Bowl with Avocado & Curried Balsamic Vinaigrette",
            self.harvester_class.title(),
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Purple Carrot")

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://images.purplecarrot.com/uploads/product/image/1357/_1400_700_Vegan_TB12_RoastedCauli_Hero-4-fcfead9e8b8f0df11875ae6baae25ab0.jpg",
            self.harvester_class.image(),
        )

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_ingredients(self):
        self.assertEqual(
            [
                "¾ cup brown lentils",
                "6 oz cauliflower florets",
                "1 tbsp curry powder",
                "2 mini sweet peppers",
                "4 oz red grapes",
                "1 avocado",
                "2 tbsp balsamic vinaigrette",
                "4 oz Arcadian greens",
                "1 tsp vegetable oil*",
                "Salt and pepper*",
                "*Not included",
                "For full ingredient list, see Nutrition",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "1 - Start the lentils. 2 - Roast the cauliflower . 3 - Prepare the toppings. 4 - Make the curried balsamic vinaigrette. 5 - Make the salad. 6 - Serve.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

import re

from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class WeightWatchers(AbstractScraper):
    @classmethod
    def host(cls):
        return "www.weightwatchers.com"

    def author(self):
        return "WeightWatchers"

    def title(self):
        return self.soup.find("h1").get_text().strip()

    def category(self):
        return "WeightWatchers"

    # cooking times, yield, difficulty are in a common div in public and non-public recipes
    # but class of that block and sub elements are different
    # so finding the block and extracting a value will be overridden in class for public recipes,
    # but picking the data item based on order is don in this base class (total_time(), cook_time() and so on)
    def _find_data_container(self):
        return self.soup.find("div", {"class": "styles_container__3N3E8"})

    def _extract_item_field(self, item):
        return item.contents[1]

    def total_time(self):
        return get_minutes(
            self._extract_item_field(self._find_data_container().contents[0])
        )

    def prep_time(self):
        return get_minutes(
            self._extract_item_field(self._find_data_container().contents[1])
        )

    def cook_time(self):
        return get_minutes(
            self._extract_item_field(self._find_data_container().contents[2])
        )

    def yields(self):
        return get_yields(
            self._extract_item_field(self._find_data_container().contents[3])
        )

    def difficulty(self):
        return self._extract_item_field(
            self._find_data_container().contents[4]
        ).get_text()

    #   Alternative way to extract data based on description instead of position
    #    def total_time(self):
    #        return get_minutes(
    #            self.__find_data_container()
    #            .find("div", string=re.compile(r"minutes Total Time"))
    #            .previous_sibling
    #        )

    def image(self):
        background_image_style = self.soup.find(
            "div", {"class": "styles_image__2dnNm"}
        )["style"]

        if background_image_style:
            return (
                re.search(r'url\("(?P<imgurl>\S*)"\);', background_image_style)
                .groupdict()
                .get("imgurl")
            )

        return None

    def _find_ingredient_tags(self):
        return self.soup.find(
            "h3", {"id": "food-detail-recipe-ingredients-header"}
        ).parent.find_all("div", {"class": "styles_name__1OYVU"})

    @staticmethod
    def _extract_ingredient_name(ingredient):
        return normalize_string(
            ingredient.find("div", {"class": "styles_ingredientName__1Vffd"})
            .find("div")
            .get_text()
        )

    @staticmethod
    def _extract_portion_parts(ingredient):
        tags = ingredient.find("div", {"class": "styles_portion__2NQyq"}).find_all(
            "span"
        )
        try:
            return (
                normalize_string(tags[0].get_text()),
                normalize_string(tags[1].get_text()),
                normalize_string(tags[2].get_text().replace(", ", ""))
                if tags[2]
                else None,
            )
        except IndexError:
            return (
                normalize_string(tags[0].get_text()),
                normalize_string(tags[1].get_text()),
                None,
            )

    def __parse_ingredient(self, ingredient):
        ingredient_name = self.__class__._extract_ingredient_name(ingredient)
        amount, unit, comment = self.__class__._extract_portion_parts(ingredient)

        if comment:
            return f"{amount} {unit} {ingredient_name}; {comment}"
        else:
            return f"{amount} {unit} {ingredient_name}"

    def ingredients(self):
        return [
            self.__parse_ingredient(ingredient)
            for ingredient in self._find_ingredient_tags()
        ]

    def _get_instructions(
        self, header_tag, header_attribute, header_value, instruction_tag
    ):
        instructions = self.soup.find(
            header_tag, {header_attribute: header_value}
        ).parent.find("ol")
        return "\n".join(
            [
                normalize_string(instruction.get_text())
                for instruction in instructions.find_all(instruction_tag)
            ]
        )

    def instructions(self):
        return self._get_instructions(
            "h3", "id", "food-detail-recipe-instruction-header", "div"
        )

    def description(self):
        return self.soup.find("div", {"class": "copy-1"}).get_text().strip()

    def nutrients(self):
        result = {}

        result["personal points"] = (
            self.soup.find("div", {"class": "styles_points__2gv9n"})
            .find("div", {"class": "styles_container__2p-YG"})
            .get_text()
        )

        veggiepoints = self.soup.find(
            "div", {"class": "styles_vegetableServings__2YSPy"}
        )
        if veggiepoints:
            result["positive points"] = normalize_string(
                veggiepoints.find(
                    "div", {"class": "styles_container__2p-YG"}
                ).next_sibling.get_text()
            )

        return result



from recipe_scrapers.cucchiaio import Cucchiaio
from tests import ScraperTest


class TestCucchiaioScraper(ScraperTest):

    scraper_class = Cucchiaio

    def test_host(self):
        self.assertEqual("cucchiaio.it", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.cucchiaio.it/ricetta/pesce-spada-al-miele-millefiori-pomodorini-e-patatine-novelle/",
            self.harvester_class.canonical_url(),
        )

    def test_author(self):
        self.assertEqual("Il Cucchiaio d'Argento", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Pesce spada al miele millefiori, pomodorini e patatine novelle",
            self.harvester_class.title(),
        )

    def test_image(self):
        self.assertEqual(
            "https://www.cucchiaio.it/content/cucchiaio/it/ricette/2017/10/pesce-spada-al-miele-millefiori-pomodorini-e-patatine-novelle/jcr:content/header-par/image-single.img10.jpg/1610381008015.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 fette di pesce spada di circa 200 g l'una",
                "20 patatine novelle",
                "1 spicchio d'aglio",
                "1 grappolo di pomodorini",
                "2 cucchiai di pinoli tostati",
                "4-5 cucchiaini di miele millefiori",
                "1 bicchiere di aceto di vino",
                "foglioline di mirto",
                "prezzemolo",
                "erba cipollina",
                "olio extravergine di oliva",
                "sale",
                "pepe",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Iniziate la preparazione del pesce spada al miele millefiori, pomodorini e patatine novelle mettendo a marinare le fette di pesce. Poggiatele su un piatto, aggiungete alcune foglie di mirto, l'aglio a fettine, un giro d'olio e lasciatele marinare per dieci minuti. In un tegame scaldate tre cucchiai d'olio, adagiatevi le fette di pesce spada e cuocetele per circa 2 minuti per lato o comunque fino a quando si forma una crosticina. Trasferitele su una teglia e passatele per circa 5 minuti in forno a 200°. Sfornatele e fatele riposare al caldo per una decina di minuti.\nIntanto tagliate a cubetti i pomodorini, cospargeteli con un po' di sale, un pizzico di pepe, prezzemolo ed erba cipollina tritati finemente. Preparate la salsina al miele: in una casseruola scaldate l'aceto facendolo ridurre di un quarto, aggiungete il miele millefiori e assaggiate per controllarne il sapore, se troppo agro aggiungete altro miele. Lasciate raffreddare.\nInfine, incorporate il tutto ai pomodori, mescolate bene, aggiungete olio, pinoli, regolate di sale e fate riposare. Sbollentate le patatine in acqua salata, asciugatele e insaporitele in una padella con un filo d’olio.\nDisponete le fette di pesce spada sul piatto da portata: completate con le patatine e cospargete su tutto la salsina al miele.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.0, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper


class KuchniaDomowa(AbstractScraper):
    @classmethod
    def host(cls):
        return "kuchnia-domowa.pl"

    def title(self):
        return self.soup.find("h2").get_text().strip()

    def image(self):
        urls = self.soup.findAll("img", {"class": "article-img", "id": "article-img-1"})
        return f"https:{urls[1]['src']}"

    def instructions(self):
        instructions = self.soup.find("div", {"id": "recipe-instructions"}).findAll(
            "li"
        )
        instructions = [x.text for x in instructions]
        return "\n".join(instructions)



from recipe_scrapers.finedininglovers import FineDiningLovers
from tests import ScraperTest


class TestFineDiningLoversScraper(ScraperTest):

    scraper_class = FineDiningLovers

    def test_host(self):
        self.assertEqual("finedininglovers.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.finedininglovers.com/recipes/main-course/zucchini-raw-vegan-lasagna",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Zucchini Raw Vegan Lasagna")

    def test_total_time(self):
        self.assertEqual(50, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "Zucchini",
                "Basil",
                "Pine Nuts",
                "Extra virgin olive oil",
                "Yeast Flakes",
                "Garlic",
                "Salt",
                "Tomatoes",
                "Sun Dried Tomatoes",
                "Extra virgin olive oil",
                "Salt",
                "Pepper",
                "Brown sugar",
                "Oregano",
                "Macadamia Nuts",
                "Yeast Flakes",
                "Salt",
                "Limes",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "For the macadamia cheese Pour all ingredients in a blender and mix until a thick cream. Set aside. For the tomato cream Pour all ingredients into a blender and mix until creamy. Set aside. For the basil pesto Pour all ingredients into a blender and mix until creamy. Set aside. Wash zucchini and cut them into very thin slices. Make the raw vegan lasagna alternating a layer of zucchini\na layer of macadamia cheese\na layer of zucchini\na layer of tomato sauce\na layer of zucchini\na layer of basil pesto. Keep the lasagna in the fridge and serve decorated with fresh basil and pine nuts.",
            self.harvester_class.instructions(),
        )

    def test_image(self):
        return self.assertEqual(
            "https://www.finedininglovers.com/sites/g/files/xknfdk626/files/Original_2242_zucchini-lasagna-raw-vegan.jpg",
            self.harvester_class.image(),
        )



from recipe_scrapers.amazingribs import AmazingRibs
from tests import ScraperTest


class TestAmazingRibsScraper(ScraperTest):

    scraper_class = AmazingRibs

    def test_host(self):
        self.assertEqual("amazingribs.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://amazingribs.com/tested-recipes/sausage-recipes/texas-hot-guts-smoked-sausage/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual("Texas Hot Guts Recipe", self.harvester_class.title())

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Meathead")

    def test_total_time(self):
        self.assertEqual(165, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 teaspoons whole black peppercorns",
                "2 teaspoons fresh ground black pepper",
                "1 tablespoon mild American paprika",
                "2 teaspoons garlic powder",
                "2 teaspoons Morton Coarse Kosher Salt",
                "2 teaspoons rubbed sage",
                "2 teaspoons cayenne, chipotle, or other hot chile powder or flakes",
                "1 medium green jalapeño",
                "½ small onion",
                "3 garlic cloves",
                "20 ounces ground pork butt",
                "12 ounces ground beef chuck",
                "⅓ cup very cold water",
                "4 feet pork sausage casings",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            [
                "Prep. Put the whole black peppercorns into a plastic bag and smash the heck outta them with a small frying pan until you have chunks of cracked peppercorns. Mix them with the rest of the black pepper, paprika, garlic powder, salt, sage, and chile powder in a small bowl. Remove the seeds and stems from the jalapeño and mince it into tiny bits. Peel the onion and garlic and mince them too. Now, go to our article on the Science of Making Sausage and follow steps (1) through (16).",
                "Smoke. Set up your grill or smoker and maintain a steady 225°F (107.2°C). Smoke the sausages at 225°F (107.2°C) until they hit 160°F (71.1°C) internal temperature, about 1 to 2 hours. As long as they hit that internal temp, you can experiment with the time to get your preferred level of smoke on the sausage.",
                "Serve. You can serve Hot Guts nekkid on a plate with some saltine crackers and hot sauce (traditional Texas style) or with some potatoes and a salad, or on a bun, or incorporate them into a dish like German Potato Salad or Choucroute Garnie, the classic Alsatian hot dish of sauerkraut, potatoes, various charcuterie, and mustard.",
            ],
            self.harvester_class.instructions_list(),
        )



# mypy: disallow_untyped_defs=False
import functools
import logging

from recipe_scrapers.settings import settings

from .._utils import normalize_string
from ._interface import PluginInterface

logging.basicConfig()
logger = logging.getLogger(__name__)


class NormalizeStringPlugin(PluginInterface):
    """
    Explicitly run the output from the methods listed through normalize_string
    """

    decorate_hosts = ("*",)
    run_on_methods = ("title",)

    @classmethod
    def run(cls, decorated):
        @functools.wraps(decorated)
        def decorated_method_wrapper(self, *args, **kwargs):
            # TODO: write logging. Configure logging.
            logger.setLevel(settings.LOG_LEVEL)
            class_name = self.__class__.__name__
            method_name = decorated.__name__
            logger.debug(
                f"Decorating: {class_name}.{method_name}() with NormalizeStringPlugin"
            )

            return normalize_string(decorated(self, *args, **kwargs))

        return decorated_method_wrapper



from recipe_scrapers.bakingmischeif import BakingMischeif
from tests import ScraperTest


class TestBakingMischeifScraper(ScraperTest):

    scraper_class = BakingMischeif

    def test_host(self):
        self.assertEqual("bakingmischief.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Tracy", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Barbacoa Burrito Bowls", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(190, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://bakingmischief.com/wp-content/uploads/2020/12/barbacoa-burrito-bowls-image-square-2.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 boneless 2-3 pound chuck roast ( trimmed and cut into fist-size chunks)",
                "Salt and pepper",
                "1 medium white or yellow onion diced",
                "3 tablespoons vegetable oil",
                "1-6 cups beef or chicken broth",
                "1/4 cup lime juice",
                "1/4 cup apple cider vinegar",
                "2 chipotle peppers in adobo sauce minced (optional)",
                "1 tablespoon ground cumin",
                "1 tablespoon dried oregano",
                "1/4 teaspoon ground cloves optional",
                "3/4 teaspoon salt",
                "3 bay leaves",
                "Juice from 1 lime (about 2 tablespoons)",
                "Scant 2 cups chicken broth",
                "1 cup long-grain rice",
                "Zest from 1 lime (divided)",
                "1 tablespoon (14g) butter (salted or unsalted is fine)",
                "1/4 teaspoon salt",
                "1/4 cup chopped loosely packed cilantro (divided)",
                "1 15-ounce can black beans (rinsed, drained, and warmed)",
                "1 15- 15-ounce can corn (drained and warmed)",
                "1/2 cup (2oz) shredded Mexican-blend cheese",
                "1 cup pico de gallo",
                "1/2 cup guacamole",
                "1/4 cup sour cream",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Barbacoa\nGenerously salt and pepper meat on all sides.\nIn a large dutch oven (or skillet if you’ll be using a slow cooker), heat oil over medium-high heat. Brown meat on all sides, in batches if necessary. This may take up to 15 minutes.\nStovetop Barbacoa\nAdd onion and just enough chicken or beef broth to mostly submerge the meat. Stir, scraping the bottom of the pan to remove any stuck-on bits. Stir in remaining ingredients and bring mixture to a low simmer. Cook, stirring occasionally for 2 1/2 to 3 hours, until meat is very tender and can be pulled apart with a fork.\nSlow Cooker Barbacoa\nIf using a slow cooker, transfer meat and any drippings remaining in the pan to the slow cooker and add onions, 1 cup of broth and remaining ingredients. Cook on high for 3 to 4 hours, low for 7 to 8 hours, until the beef is tender and can be easily shredded with a fork.\nShred\nOnce meat is done, use a slotted spoon to transfer the pieces to a cutting board. Use two forks to shred the meat, discarding any large pieces of fat as you go.\nReturn shredded meat to the cooking liquid. Add more salt and pepper to taste.\nCilantro Lime Rice\nAdd lime juice to a 2-cup measuring cup. Fill the measuring cup the rest of the way with chicken broth.\nIn a saucepan or rice cooker, cook rice according to package instructions, replacing the water with the lime juice/chicken broth mixture and adding butter and salt when you add the rice (if the package instructions already call for salt, do not add additional salt).\nAfter rice has finished cooking, stir in half of the lime zest and cilantro. Taste and add more lime and cilantro to taste.\nBurrito Bowls\nDivide cilantro rice between bowls or food prep containers. Top rice with shredded barbacoa and the rest of your burrito bowl toppings. Serve and enjoy.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.bbcgoodfood import BBCGoodFood
from tests import ScraperTest


class TestBBCGoodFoodScraper(ScraperTest):

    scraper_class = BBCGoodFood

    def test_host(self):
        self.assertEqual("bbcgoodfood.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.bbcgoodfood.com/recipes/monster-cupcakes",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Monster cupcakes")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Good Food team")

    def test_total_time(self):
        self.assertEqual(50, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("12 items", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://images.immediate.co.uk/production/volatile/sites/30/2020/08/recipe-image-legacy-id-405483_12-cee017a.jpg?resize=768,574",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "250g self-raising flour",
                "25g cocoa powder",
                "175g light muscovado sugar",
                "85g unsalted butter , melted",
                "5 tbsp vegetable or sunflower oil",
                "150g pot fat-free natural yogurt",
                "1 tsp vanilla extract",
                "3 large eggs",
                "85g unsalted butter , softened",
                "1 tbsp milk",
                "½ tsp vanilla extract",
                "200g icing sugar , sifted",
                "food colourings (optional)",
                "sweets and sprinkles, to decorate",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Heat oven to 190C/170C fan/gas 5 and line a 12-hole muffin tin with deep cake cases. Put all the cake ingredients into a large bowl and beat together with electric hand beaters until smooth. Spoon the mix into the cases, then bake for 20 mins until risen and a skewer inserted into the middle comes out dry. Cool completely on a rack. Can be made up to 3 days ahead and kept in an airtight container, or frozen for up to 1 month.\nFor the frosting, work the butter, milk and vanilla into the icing sugar until creamy and pale. Colour with food colouring, if using, then create your own gruesome monster faces using sweets and sprinkles.",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            "Let your little monsters do their worst, decorating these spooky Halloween treats",
            self.harvester_class.description(),
        )



# mypy: disallow_untyped_defs=False
import functools
import logging

from recipe_scrapers.settings import settings

from ._interface import PluginInterface

logging.basicConfig()
logger = logging.getLogger(__name__)


class SchemaOrgFillPlugin(PluginInterface):
    """
    If any of the methods listed is invoked on a scraper class
    that happens not to be implement and Schema.org is available
    attempt to return the result from the schema available.
    """

    run_on_hosts = ("*",)
    run_on_methods = (
        "author",
        "title",
        "category",
        "total_time",
        "yields",
        "image",
        "ingredients",
        "instructions",
        "ratings",
        "reviews",
        "links",
        "language",
        "nutrients",
    )

    @classmethod
    def run(cls, decorated):
        @functools.wraps(decorated)
        def decorated_method_wrapper(self, *args, **kwargs):
            logger.setLevel(settings.LOG_LEVEL)
            class_name = self.__class__.__name__
            method_name = decorated.__name__
            logger.debug(
                f"Decorating: {class_name}.{method_name}() with SchemaOrgFillPlugin"
            )
            try:
                return decorated(self, *args, **kwargs)
            except NotImplementedError as e:
                function = getattr(self.schema, decorated.__name__)
                if self.schema.data and function:
                    logger.info(
                        f"{class_name}.{method_name}() seems to not be implemented but .schema is available! Attempting to return result from .schema."
                    )
                    return function(*args, **kwargs)
                else:
                    raise e

        return decorated_method_wrapper



from recipe_scrapers.kwestiasmaku import KwestiaSmaku
from tests import ScraperTest


class TestKwestiaSmakuScraper(ScraperTest):

    scraper_class = KwestiaSmaku

    def test_host(self):
        self.assertEqual("kwestiasmaku.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("kwestiasmaku", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Pieczony kalafior", self.harvester_class.title())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.kwestiasmaku.com/sites/v123.kwestiasmaku.com/files/pieczony-kalafior-01_0.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 kalafior",
                "4 łyżki oliwy z pierwszego tłoczenia",
                "świeżo zmielony pieprz",
                "1 i 1/2 łyżeczki kurkumy",
                "1 łyżeczka czarnuszki",
                "sól morska",
                "sos czosnkowy lub ser feta",
                "natka pietruszki",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Odciąć liście z kalafiora, przekroić na pół i wyciąć nadmiar głąba. Główkę kalafiora rozdzielić na różyczki lub pokroić na cząstki.\n"
            "Położyć na dużej blaszce (np. z wyposażenia piekarnika) wyłożonej papierem do pieczenia. Piekarnik nagrzać do 200 stopni C.\n"
            "Kalafiora polać oliwą, posypać pieprzem, kurkumą i czarnuszką, a następnie dokładnie wymieszać i rozłożyć na całej powierzchni blachy.\n"
            "Wstawić do nagrzanego piekarnika i piec przez 30 minut. Po upieczeniu doprawić solą i posiekaną natką pietruszki.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.83, self.harvester_class.ratings())



from recipe_scrapers.giallozafferano import GialloZafferano
from tests import ScraperTest


class TestGialloZafferanoScraper(ScraperTest):

    scraper_class = GialloZafferano

    def test_host(self):
        self.assertEqual("ricette.giallozafferano.it", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://ricette.giallozafferano.it/Bavarese-alle-fragole.html",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Bavarese alle fragole")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "GialloZafferano")

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("10 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "Latte intero 200 ml",
                "Tuorli 3",
                "Zucchero 105 g",
                "Baccello di vaniglia ½",
                "Limoni ½",
                "Sale fino 1 pizzico",
                "Fragole 1.1 kg",
                "Gelatina in fogli 18 g",
                "Panna fresca liquida 500 ml",
                "Panna fresca liquida 150 ml",
                "Zucchero a velo 30 g",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Per preparare la bavarese alle fragole, iniziate dalla crema inglese che è la base del dolce: incidete mezza bacca di vaniglia e raschiate con la punta di un coltellino per estrarre i semini 1 . Prelevate la scorza del limone facendo attenzione a non sbucciare anche la parte bianca 2 e unite i due aromi nel pentolino con il latte 3 : scaldate a fuoco dolce fino a sfiorare il bollore poi spegnete il fuoco.\nIntanto versate i tuorli in una ciotola capiente sbatteteli con la frusta poi aggiungete 65 g di zucchero semolato a pioggia continuando a mescolare 4 e aggiungete un pizzico di sale 5 . Lavorateli per qualche minuto, fino ad ottenere un composto omogeneo. Versate sulle uova sbattute il latte scaldato, filtrandolo con un colino a maglie strette 6 .\nMescolate con una spatola 7 e ponete la crema a cuocere a bagno maria a fuoco basso, sempre mescolando 8 ; la temperatura della crema non deve mai superare gli 85° (controllate con un termometro da cucina) 9 .\nQuando la crema sarà pronta versatela in una ciotola e ponete la ciotola in un'altra più capiente e piena di ghiaccio 10 : in questo modo rafferdderete immediatamente la crema senza che la consistenza ne risenta. Mescolate 11 poi togliete dal ghiaccio e coprite la crema con pellicola a contatto per farla raffreddare completamente. Mettete in ammollo i fogli di gelatina in acqua molto fredda 12 per almeno 10 minuti.\nLavate, scolate per bene e togliete il picciolo alle fragole, quindi tagliatele a pezzi 13 e cuocetele in padella con 40 g di zucchero semolato a fuoco dolce, coperte con un coperchio (14-15).\nFrullate le fragole cotte con lo zucchero 16 e setacciate la purea ottenuta 17 . Quindi trasferitela in un pentolino e rimettetela sul fuoco dolce 18 .\nScolate e strizzate i fogli di gelatina e scioglieteli nella coulis di fragole 19 , mescolate con una frusta per farli sciogliere completamente 20 . Fate intiepidire il composto, e unitelo alla crema inglese 21 e mescolate.\nPoi a parte montate 500 ml di panna fresca (non troppo ferma) 22 e unitela alla crema di fragole 23 e amalgamate bene il composto: dovrà risultare fluido, cremoso e omogeneo. Quindi prendete uno stampo da ciambella e bagnate l'interno con dell'acqua fredda. Versate delicatamente il composto ottenuto nello stampo di 22 cm di diametro e capienza 1,5 lt 24 .\nCopritelo con pellicola trasparente 25 e ponetelo in frigorifero per almeno 4-6 ore (meglio sarebbe prepararlo la sera prima). Preparate poi la panna montata con lo zucchero a velo 26 e ponetela in una sac-à-poche 27 e in frigorifero fino al momento dell'utilizzo.\nTrascorso il tempo necessario, estraete il bavarese dal frigorifero, immergete lo stampo fino al bordo in acqua calda per qualche secondo 28 e poi capovolgete delicatamente il bavarese su di un piatto da portata 29 . Decorate il bavarese a piacere con ciuffetti di panna tutto intorno 30 .\nCreate dei ciuffi anche sulla superficie 31 . Poi decorate con le fragole fresche e foglioline di menta a piacere (32-33).",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.0, self.harvester_class.ratings())



from recipe_scrapers.timesofindia import TimesOfIndia
from tests import ScraperTest


class TestTimesOfIndiaScraper(ScraperTest):

    scraper_class = TimesOfIndia

    def test_host(self):
        self.assertEqual("recipes.timesofindia.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("TNN", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Stuffed Tortellini Recipe", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://static.toiimg.com/thumb/71330969.cms?width=1200&height=900",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "500 gm cherry tomatoes",
                "1 teaspoon spice oregano",
                "black pepper as required",
                "2 cloves garlic",
                "1/4 cup parmesan cheese",
                "water as required",
                "4 tablespoon extra virgin olive oil",
                "salt as required",
                "250 gm pasta tortelleni",
                "2 teaspoon pesto sauce",
                "6 basil",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "To prepare this palatable dish, preheat the oven at 200 degrees Celsius. Meanwhile, spread a baking sheet, place cherry tomatoes topped with a drizzle of 2 tablespoons of extra virgin olive oil. Season the tomatoes with salt, black pepper, and oregano spice. Make sure the tomatoes are evenly coated with the seasoned oil by rolling them over and over inside the baking sheet itself.\nBy now, the oven should be ready to roast. Place the baking sheet with tomatoes inside the oven and roast the tomatoes for 15 minutes or until they start to burst out.\nWhile the tomatoes are roasting in the oven, put a large pan over medium flame and boil water in it. Add the tortellini pasta with a pinch of salt and a little olive oil. Once cooked, drain the water and keep the pasta aside until required.\nNow, heat 2 tablespoons of extra virgin olive oil in a skillet over low-medium heat. After the oil turns hot, add minced garlic and saute until it turns golden brown. Turn off the flame and to the garlic, add roasted tomatoes and pesto sauce. Using a fork, stab the tomatoes to break them open. Stir the mixture and then add cooked tortellini. Pour some more pesto sauce over the tortellini and continue stirring. Sprinkle shredded parmesan cheese and garnish with torn or chopped basil leaves.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())

    def test_language(self):
        self.assertEqual("en", self.harvester_class.language())

    def test_cuisine(self):
        self.assertEqual("Italian", self.harvester_class.cuisine())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import normalize_string


class TasteOfHome(AbstractScraper):
    @classmethod
    def host(cls):
        return "tasteofhome.com"

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        instructions = self.soup.findAll("li", {"class": "recipe-directions__item"})
        if instructions:
            return "\n".join(
                [
                    normalize_string(instruction.get_text())
                    for instruction in instructions
                ]
            )
        else:
            # In case our HTML parsing doesn't find any instructions, fall back to what the schema provides.
            return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()



from recipe_scrapers.tastesbetterfromscratch import TastesBetterFromScratch
from tests import ScraperTest


class TestTastesBetterFromScratch(ScraperTest):

    scraper_class = TastesBetterFromScratch

    def test_host(self):
        self.assertEqual("tastesbetterfromscratch.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://tastesbetterfromscratch.com/pulled-pork/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "How to make Pulled Pork")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Lauren Allen")

    def test_total_time(self):
        self.assertEqual(320, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://tastesbetterfromscratch.com/wp-content/uploads/2020/05/Pulled-Pork-5.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertSetEqual(
            set(
                [
                    "4 lb pork shoulder (, or butt)",
                    "2 Tablespoons oil (optional if searing)",
                    "1 Tbsp brown sugar",
                    "1 tablespoon chili powder",
                    "1 teaspoon onion powder",
                    "1 teaspoon garlic powder",
                    "1 teaspoon cumin",
                    "1 teaspoon kosher salt",
                    "1 teaspoon black pepper",
                    "12 ounces coke (not diet)",
                    "bbq sauce for coating meat (optional)",
                ]
            ),
            set(self.harvester_class.ingredients()),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "Trim pork of excess fat and cut into 4 pieces.",
                    "Combine spices in a small bowl and rub all over the pork. (This can be done the night before).",
                    "Sear (optional): Heat a few tablespoons of oil in a Dutch oven pot over medium-high heat. Add the meat and sear for a few seconds on all sides.",
                    "Oven Method: Preheat oven to 300 degrees F. Pour coke around the pork in the Dutch oven pot. Cover pot with lid and cook for 3 hours. Remove lid and cook for an additional 1-2 hours, until pork is tender and easily pulls apart with a fork. Remove from oven and shred meat. Toss in barbecue sauce, if desired.",
                    "Slow Cooker Method: Place pork in slow cooker and pour coke around it. Cover and cook on LOW (recommended) 8 hours or high for 4-5 hours, until pork is tender and shreds easily with a fork.",
                    "Instant Pot Method: Place pork in instant pot and pour coke around it. Cook on Manual/High pressure for 70 minutes. When timer beeps, allow the pot to naturally release pressure, about 15 minutes longer. Remove lid and shred the meat.",
                ]
            ),
            self.harvester_class.instructions(),
        )



from recipe_scrapers.akispetretzikis import AkisPetretzikis
from tests import ScraperTest


class TestAkisPetretzikisScraper(ScraperTest):

    scraper_class = AkisPetretzikis

    def test_host(self):
        self.assertEqual("akispetretzikis.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Akis Petretzikis", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Lemon chicken with artichokes", self.harvester_class.title())

    def test_category(self):
        self.assertEqual(None, self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8-10", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://d3fch0cwivr6nf.cloudfront.net/system/uploads/medium/data/15973/aginares-lemonates-me-kotopoulo.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 1/2 kilo chicken breast fillet",
                "4-5 tablespoon(s) olive oil",
                "salt",
                "pepper",
                "50 g all-purpose flour",
                "2 onions",
                "2 clove(s) of garlic",
                "750 g artichokes",
                "80 g white wine",
                "lemon juice",
                "400 g water",
                "1 tablespoon(s) chicken stock pot",
                "2 tablespoon(s) tarragon",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "Place a frying pan over high heat and add 2-3 tablespoons olive oil.",
                    "Cut the chicken into 2-3 cm pieces and add them into a bowl. Add salt, pepper, the flour, and mix.",
                    "Transfer the chicken to the hot pan and sauté for 3-4 minutes until golden.",
                    "Place a pot over high heat and add 2 tablespoons olive oil.",
                    "Finely chop the onions and the garlic, add them to the pot, and sauté.",
                    "Add the artichokes, the chicken, and deglaze the pot with the wine.",
                    "Add the lemon juice, the water, the chicken stock pot, the tarragon, and mix.",
                    "Cover with the lid and simmer over medium heat for 20-25 minutes.",
                    "Serve with pepper, olive oil, lemon slices, and parsley.",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.91, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual(None, self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Lemon chicken with artichokes by the Greek chef Akis Petretzikis. A quick and easy recipe for a hearty lemon chicken with tender artichokes!",
            self.harvester_class.description(),
        )

    def test_language(self):
        self.assertEqual("en", self.harvester_class.language())



# mypy: disallow_untyped_defs=False

from ._abstract import AbstractScraper
from ._utils import normalize_string


class Reishunger(AbstractScraper):
    @classmethod
    def host(cls):
        return "reishunger.de"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        # find the "instructions" heading (Zubereitung in German)
        for heading in self.soup.findAll("h3"):
            if "Zubereitung" in heading.get_text():
                break

        results = []

        # locate the first recipe instruction
        step1 = heading.parent.parent.find("div", {"class": "leading-normal"})

        # iterate through each step in the recipe
        for step in step1.next_siblings:

            # check whether the instruction has a list of preparations
            # fixme: this can throw an exception if 'step' is not a bs4 Tag
            try:
                preparations = step.find("div", {"preparation": True})
            except Exception:
                preparations = None

            # if it does, add every preparation step as an instruction entry
            if preparations:
                for preparation in preparations.findAll("div", {"id": True}):
                    instruction = normalize_string(preparation.text)
                    results.append(instruction)

            # otherwise, add only one instruction entry
            else:
                if step.find("p"):
                    instruction = normalize_string(step.text)
                    results.append(instruction)

            # continue on to the next instruction
            step = step.next_sibling

        # filter out empty lines
        results = [instruction for instruction in results if instruction]
        return "\n".join(results)

    def ratings(self):
        return self.schema.ratings()



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class WikiCookbook(AbstractScraper):
    @classmethod
    def host(cls):
        return "en.wikibooks.org"

    def title(self):
        return self.soup.find("h1").get_text().replace("Cookbook:", "")

    def total_time(self):
        return get_minutes(self.soup.find("th", string="Time").find_next_sibling("td"))

    def yields(self):
        return get_yields(
            self.soup.find("th", string="Servings").find_next_sibling("td")
        )

    def image(self):
        image = self.soup.find("a", {"class": "image"}).find("img", {"src": True})
        return image["src"] if image else None

    def ingredients(self):
        ingredients = (
            self.soup.find("span", {"id": "Ingredients"}).find_next("ul").findAll("li")
        )

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = (
            self.soup.find("span", {"id": "Procedure"}).find_next("ol").findAll("li")
        )

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )

    def description(self):
        paragraphs = list()
        for tag in self.soup.find(class_="mw-parser-output"):
            try:
                # get all paragraphs except for links
                if (
                    tag.text.strip()
                    and tag.name == "p"
                    and not tag.find("span", {"id": "displaytitle"})
                ):
                    paragraphs.append(tag)
                # End at the TOC or second section
                if tag.attrs.get("id") == "toc" or tag.name == "h2":
                    break
            except AttributeError:
                # Ignore tags that are not <p> but raise errors
                pass

        return "\n\n".join(
            [normalize_string(paragraph.get_text()) for paragraph in paragraphs]
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class Woop(AbstractScraper):
    @classmethod
    def host(cls):
        return "woop.co.nz"

    def title(self):
        found = self.soup.find("meta", {"property": "og:title"})
        return normalize_string(found["content"])

    def ingredients(self):
        div = self.soup.findAll("div", {"class": "ingredients"})[0]
        li = div.findChildren("li")
        return [normalize_string(i.text) for i in li if i.text]

    def yields(self):
        div = self.soup.findAll("div", {"class": "serving-amount"})[0]
        value = div.findChildren("div", {"class": "value"})[0]
        return get_yields(value.text)

    def total_time(self):
        div = self.soup.findAll("div", {"class": "cook-time"})[0]
        p = div.findChildren("p")[0]
        value = get_minutes(p)
        return value

    def instructions(self):
        div = self.soup.findAll("div", {"class": "cooking-instructions"})[0]
        li = div.findChildren("li")
        normalized = [normalize_string(i.text) for i in li]
        return "\n".join([i for i in normalized if i])

    def nutrients(self):
        div = self.soup.findAll("div", {"class": "nutritional-info"})[0]
        li = div.findChildren("li")
        return {
            normalize_string(nutrient): normalize_string(value)
            for nutrient, value in [i.text.split(":") for i in li]
            if value is not None
        }



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class TastyKitchen(AbstractScraper):
    @classmethod
    def host(cls):
        return "tastykitchen.com"

    def title(self):
        return self.soup.find("h1", {"itemprop": "name"}).get_text()

    def total_time(self):
        return sum(
            [
                get_minutes(self.soup.find("time", {"itemprop": "prepTime"})),
                get_minutes(self.soup.find("time", {"itemprop": "cookTime"})),
            ]
        )

    def yields(self):
        return get_yields(self.soup.find("span", {"itemprop": "yield"}))

    def image(self):
        image = self.soup.find("img", {"class": "the_recipe_image", "src": True})
        return image["src"] if image else None

    def ingredients(self):
        ingredients = self.soup.find("ul", {"class": "ingredients"}).findAll("li")

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = self.soup.find("span", {"itemprop": "instructions"}).findAll("p")

        return "\n".join(
            [normalize_string(direction.get_text()) for direction in instructions]
        )



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class SunBasket(AbstractScraper):
    @classmethod
    def host(cls, domain="com"):
        return f"sunbasket.{domain}"

    def title(self):
        return self.soup.find("h1").get_text()

    def total_time(self):
        minutes_tag = self.soup.find("span", string=re.compile(r"Minutes"))
        return get_minutes(minutes_tag.parent.get_text())

    def yields(self):
        yields_tag = self.soup.find("span", string=re.compile(r"Servings,"))
        return get_yields(yields_tag.parent.get_text())

    def ingredients(self):
        ingredients_container = self.soup.find(class_="ingredients-list")
        ingredients = ingredients_container.findAll("li")

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def _instructions_list(self):
        instructions_container = self.soup.find(
            "div", {"class": "instructions-container"}
        )
        instructions = instructions_container.findAll("div", {"class": "step"})
        instruction_list = []
        for instruction in instructions:
            step_number_tag = instruction.find(class_="step-number")
            if step_number_tag is not None:
                step_number = normalize_string(
                    instruction.find(class_="step-number").get_text()
                )
                step_name = normalize_string(
                    instruction.find(class_="step-header").get_text()
                )
                step_instructions = normalize_string(
                    instruction.find(class_="instruction-description").get_text()
                )
                instruction_list.append(
                    f"{step_number}: {step_name} - {step_instructions}"
                )
        return instruction_list

    def instructions(self):
        data = self._instructions_list()
        return "\n".join(data) if data else None

    def image(self):
        container = self.soup.find("div", {"class": "recipe-image-container"})
        if not container:
            return None

        image = container.find("img", {"src": True})
        return image["src"] if image else None



from recipe_scrapers.imworthy import ImWorthy
from tests import ScraperTest


class TestImWorthyScraper(ScraperTest):

    scraper_class = ImWorthy

    def test_host(self):
        self.assertEqual("im-worthy.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Shana Thomas", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Vegan Almond Flour Pancakes (Fluffy & Gluten-free)",
            self.harvester_class.title(),
        )

    def test_category(self):
        self.assertEqual(
            "Breakfast,Brunch,Main Course", self.harvester_class.category()
        )

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.im-worthy.com/wp-content/uploads/2020/11/Almond-Flour-Pancakes_Blog4.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1/2 cup almond flour",
                "1 cup *oat flour (or use all-purpose flour)",
                "1/2-1 cup non-dairy milk (plus more, if needed)",
                "2 tsp baking powder",
                "1/2 tsp cinnamon (optional) (can sub for vanilla extract)",
                "pinch of salt",
                "1 tbsp unsweetened apple sauce or maple syrup (optional)",
                "1 tsp grapeseed oil (for cooking. Add more, if needed. Can sub for coconut or avocado oil)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "In separate bowls: mix the wet & dry ingredients.\nAdd the wet ingredients to the dry ingredients and stir until combined. Start with 1/2 cup milk and add more until the batter is slightly lumpy & creamy. (It should not be runny. If runny, add more oat/all-purpose flour. If too thick, add more non-dairy milk.)\nLet the pancake batter rest for 5 minutes to thicken while you heat the pan according to the next steps below.\nAdd the oil to a nonstick pan over medium heat. Once the pan is hot pour the batter into the pan using a 1/4 measuring cup (for easy clean up).\nCook on each side for about 2-5 minutes (depends on your pan) or until you see little bubbles form on top, then flip and let them cook for another couple of minutes.\nServe with your favorite toppings. Some suggestions include maple syrup, coconut whip cream, fresh berries, or vegan butter.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("American", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "These almond flour pancakes are sweet, fluffy and easy to make. They're vegan, gluten-free and can easily be made keto-friendly. Serve with fresh fruit or maple syrup.",
            self.harvester_class.description(),
        )



from recipe_scrapers.epicurious import Epicurious
from tests import ScraperTest


class TestEpicurious(ScraperTest):

    scraper_class = Epicurious

    def test_host(self):
        self.assertEqual("epicurious.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://test.example.com/epicurious.testhtml_files/ramen-noodle-bowl-with-escarole-and-spicy-tofu-crum_002.html",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Ramen Noodle Bowl with Escarole and Spicy Tofu Crumbles",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Claire Saffitz")

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://assets.epicurious.com/photos/568194b8fb9544f72b678fd4/master/pass/Ramen-Noodle-Bowl-With-Escarole.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 (5.5-ounce) servings fresh or dried ramen noodles",
                "4 cups torn escarole",
                "3 tablespoons Roasted Garlic Chili Sauce",
                "Kosher salt",
                "4 Pickled Scallions",
                "Spicy Tofu Crumbles, thinly sliced radish, and chopped peanuts (for serving)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            'Preparation\n\nCook noodles according to package directions. During the last minute of cooking, add escarole. Drain and rinse under cold water.\nToss noodles, escarole, and chili sauce in a large bowl until coated; season with salt. Divide noodles between bowls. Slice scallions into 1" pieces and place on top of noodles along with some tofu crumbles, radishes, and peanuts.',
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        return self.assertGreaterEqual(self.harvester_class.ratings(), 0.99)



from recipe_scrapers.nutritionbynathalie import NutritionByNathalie
from tests import ScraperTest


class TestNutritionByNathalieScraper(ScraperTest):

    scraper_class = NutritionByNathalie

    def test_host(self):
        self.assertEqual("nutritionbynathalie.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.nutritionbynathalie.com/single-post/2020/07/30/mexican-cauliflower-rice",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Mexican Cauliflower Rice")

    def test_image(self):
        self.assertEqual(
            "https://static.wixstatic.com/media/d3b5ba_7ae468273837425aa869486557b06bac~mv2.jpg/v1/fit/w_1000%2Ch_1000%2Cal_c%2Cq_80/file.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 bag fresh or frozen cauliflower rice (if using fresh cauliflower rice, add olive oil, avocado oil or coconut oil to pan)",
                "1-2 Tbsp olive oil",
                "1/4 teaspoon turmeric",
                "1/4 teaspoon cayenne pepper (optional)",
                "1/2 teaspoon garlic powder",
                "3/4 cup salsa",
                "vegan chive or scallion cream cheese",
                "fresh cilantro, chopped",
                "sea salt and pepper to taste",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Heat a pan on medium heat with oil.\nAdd the cauliflower and allow it to cook for about 5 minutes (should be nearly fully cooked).\nTurn heat down to low and add turmeric, cayenne, garlic powder, salsa, salt and pepper and continue to cook until done (about 2-3 more minutes).\nStir in vegan cream cheese and cilantro. Serve immediately and enjoy!",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.recipetineats import RecipeTinEats
from tests import ScraperTest


class TestRecipeTinEatsScraper(ScraperTest):

    scraper_class = RecipeTinEats

    def test_host(self):
        self.assertEqual("recipetineats.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.recipetineats.com/vietnamese-caramel-pork/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Vietnamese Caramel Pork")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Nagi")

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.recipetineats.com/wp-content/uploads/2017/10/Vietnamese-Caramel-Pork_-2.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1/2 cup / 100g brown sugar, tightly packed",
                "1 tbsp water",
                '1 kg / 2 lb pork shoulder ((butt) or boneless skinless pork belly, cut into 3 cm / 1.2" pieces (Note 1a))',
                "1 1/4 cups / 375 ml coconut water ((Note 1b))",
                "1 eschallot / shallot (, very finely sliced (Note 2))",
                "2 garlic cloves (, minced)",
                "1 1/2 tbsp fish sauce",
                "1/4 tsp white pepper",
                "Red chilli and finely sliced shallots/green onions",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Place sugar and water in a large pot over medium heat. Stir, then when it bubbles and the sugar is melted (it looks like caramel), add the rest of the ingredients.\nStir, then adjust the heat so it is simmering fairly energetically. Not rapidly, not a slow simmer (I use medium heat on a weak stove, between medium and low on a strong stove).\nSimmer for 1.5 hours, uncovered. Stir once or twice while cooking.\nAt around 1.5 hours, when the liquid has reduced down and the pork is tender, (see Note 3 if pork is not yet tender), the fat will separate (see video).\nStir and the pork will brown and caramelise in the fat.\nOnce the liquid is all gone and it's now stuck on the pork pieces, it's ready.\nServe over rice, garnished with fresh chilli and shallots. Simple pickled vegetables are ideal for a side because the fresh acidity pairs well with the rich pork.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper
from ._utils import normalize_string


class JustOneCookbook(AbstractScraper):
    @classmethod
    def host(cls):
        return "justonecookbook.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        lis = self.soup.find_all("li", {"class": "wprm-recipe-ingredient"})
        ingredients = []
        for ingredient in lis:
            spans = ingredient.findAll(
                "span", class_=lambda x: x != "wprm-checkbox-container"
            )[1:]
            ingredient = []
            for span in spans:
                ingredient.append(normalize_string(span.get_text()))
            ingredients.append(" ".join(ingredient))
        return ingredients

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.schema.description()



from recipe_scrapers.sunset import Sunset
from tests import ScraperTest


class TestSunsetScraper(ScraperTest):

    scraper_class = Sunset

    def test_host(self):
        self.assertEqual("sunset.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.sunset.com/recipe/crisp-top-sourdough-stuffing",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Crisp-Top Sourdough Stuffing",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Angela Brassinga")

    def test_total_time(self):
        self.assertEqual(90, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("12 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://img.sunset02.com/sites/default/files/styles/4_3_horizontal_-_900x675/public/crisp-top-sourdough-stuffing-su.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 1-pound loaf sourdough, at least 1 day old",
                "1/4 cup salted butter",
                "2 cups chopped onion (1 large)",
                "1 cup chopped celery (2 or 3 stalks)",
                "1/4 cup chopped flat-leaf parsley",
                "1 tablespoon finely chopped fresh sage",
                "About 1/2 tsp. kosher salt",
                "About 1/2 tsp. pepper",
                "About 3 cups turkey broth, reduced-sodium chicken broth, or mushroom or other vegetable broth",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Step 1\n Slice bread into 1 1/2-in.-thick slices and tear into irregular 1- to 2-in. pieces. Spread on a rimmed baking sheet and leave to dry at room temperature until needed (up to 2 days). For the best stuffing, the bread should be very dry.\nStep 2\n Preheat oven to 350°. Melt butter in a large frying pan over medium heat. Pour out 2 tbsp. butter and set aside.\nStep 3\n Add onion, celery, herbs, and 1/2 tsp. each salt and pepper to hot pan. Cook until onions are translucent and celery is tender-crisp, about 15 minutes. Transfer to a large bowl.\nStep 4\n Add torn bread and broth to vegetables and mix in until bread is soaked. Add salt and pepper to taste.\nStep 5\n Generously coat a 9- by 13-in. glass baking pan with 1 tsp. reserved melted butter. Pour stuffing into pan and drizzle with remaining melted butter.\nStep 6\n Cover with foil; bake 25 minutes. Remove foil and bake until starting to brown on top, about 30 minutes more.\nStep 7\nMake ahead: Up to 2 days, chilled. Reheat at 350°, covered, until hot (about 30 minutes). Remove foil and cook 10 more minutes for a crunchy top layer.\nStep 8\nVARIATIONS\nStep 9\nCrisp-Top Sourdough Stuffing with Sausage and Greens: Add 8 oz. sautéed crumbled Italian \xadsausage and 1 lb. briefly sautéed fresh spinach leaves to stuffing before baking.\nStep 10\nScandinavian Stuffing: Replace sourdough with a 1-lb. loaf of crusty rye bread, then add 1 cup chopped fresh dill and 8 oz. diced smoked pork chops to stuffing before baking. Top with 2 tbsp. fresh dill sprigs before serving.",
            self.harvester_class.instructions(),
        )



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper
from ._utils import normalize_string


class RosannaPansino(AbstractScraper):
    @classmethod
    def host(cls):
        return "rosannapansino.com"

    def title(self):
        return self.soup.find("meta", {"property": "og:title"})["content"]

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ingredients = (
            self.soup.find("div", {"class": "recipe-left"})
            .find_next("em", string="Ingredients")
            .find_next_sibling("ul")
            .find_all("li")
        )
        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions = (
            self.soup.find("div", {"class": "recipe-right"})
            .find_next("ol")
            .find_all("li")
        )
        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )

    def description(self):
        descriptions = self.soup.find(
            "div", {"class": "recipe-main-image-caption"}
        ).find_all("p")
        return "\n".join(
            [normalize_string(description.get_text()) for description in descriptions]
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import normalize_string


class TheHappyFoodie(AbstractScraper):
    @classmethod
    def host(cls):
        return "thehappyfoodie.co.uk"

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ingredient_elements = self.soup.find(
            "div", {"class": "hf-ingredients__container"}
        ).findAll("tr")

        amount = 0
        ingredient_name = 1
        ingredients = []
        for e in ingredient_elements:
            # Skip elements that look like section headings (for example, 'For the sauce:')
            if e.get("class"):
                continue
            ingredients.append(
                (
                    e.find_all("td")[amount].get_text(),
                    e.find_all("td")[ingredient_name].get_text(),
                )
            )

        return [
            normalize_string("{} {}".format(amount, name))
            for amount, name in ingredients
        ]

    def instructions(self):
        return self.schema.instructions()

    def author(self):
        return self.schema.author()

    def cuisine(self):
        return self.schema.cuisine()



from recipe_scrapers.eatingbirdfood import EatingBirdFood
from tests import ScraperTest


class TestEatingBirdFoodScraper(ScraperTest):

    scraper_class = EatingBirdFood

    def test_host(self):
        self.assertEqual("eatingbirdfood.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Brittany Mullins", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Easy Shakshuka", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.eatingbirdfood.com/wp-content/uploads/2020/12/Shakshuka-in-pan-225x225.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 Tablespoons olive oil",
                "1 large onion, chopped",
                "1 orange bell pepper, chopped",
                "4 cloves garlic, minced",
                "1/2 teaspoon cumin",
                "1/2 teaspoon paprika",
                "1/4 teaspoon cayenne pepper",
                "1-2 teaspoons curry powder",
                "1/4 teaspoon turmeric",
                "1/2 teaspoon sea salt",
                "1/4 teaspoon ground pepper",
                "28 ounce can diced tomatoes",
                "4-5 eggs",
                "1/2 cup feta cheese",
                "fresh cilantro and parsley, for serving",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Heat oil in a large skillet over medium heat. Add onion, bell pepper and garlic to the skillet and cook until onions are soft and fragrant — about 5 to 10 minutes.\nAdd cumin, paprika, cayenne, curry, turmeric, salt and pepper. Give the mixture a stir and cook for about 1 minute more. Add diced tomatoes to the skillet and bring sauce to a boil. Reduce heat to a simmer and cook until the sauce thickens up a bit, about 10 minutes. Add feta cheese to the tomato mixture and stir.\nCrack eggs into tomato sauce. You should be able to fit 4-5 eggs in a large skillet. Cover and let the eggs cook for about 5 minutes, or until the egg whites are completely cooked through. Remove skillet from heat, uncover and let sit for a 1-2 minutes before serving.\nSpoon 1-2 eggs along with a big serving of tomato sauce on to each plate. Garnish with extra feta cheese and fresh cilantro and parsley. Serve with toast, veggies or over a whole grain like quinoa or brown rice for a complete meal.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False

import html
import math
import re

import isodate

from ._exceptions import ElementNotFoundInHtml

FRACTIONS = {
    "¼": 0.25,
    "½": 0.50,
    "¾": 0.75,
    "⅓": 0.33,
    "⅔": 0.66,
    "⅕": 0.20,
    "⅖": 0.40,
    "⅗": 0.60,
}

TIME_REGEX = re.compile(
    r"(\D*(?P<days>\d+)\s*(days|D))?(\D*(?P<hours>[\d.\s/?¼½¾⅓⅔⅕⅖⅗]+)\s*(hours|hrs|hr|h|óra|:))?(\D*(?P<minutes>\d+)\s*(minutes|mins|min|m|perc|$))?",
    re.IGNORECASE,
)

SERVE_REGEX_NUMBER = re.compile(r"(\D*(?P<items>\d+)?\D*)")

SERVE_REGEX_ITEMS = re.compile(
    r"\bsandwiches\b |\btacquitos\b | \bmakes\b | \bcups\b | \bappetizer\b | \bporzioni\b | \bcookies\b | \b(large |small )?buns\b",
    flags=re.I | re.X,
)

SERVE_REGEX_TO = re.compile(r"\d+(\s+to\s+|-)\d+", flags=re.I | re.X)


def get_minutes(element, return_zero_on_not_found=False):  # noqa: C901: TODO
    if element is None:
        # to be removed
        if return_zero_on_not_found:
            return 0
        raise ElementNotFoundInHtml(element)

    # handle integer in string literal
    try:
        return int(element)
    except Exception:
        pass

    if isinstance(element, str):
        time_text = element
    else:
        time_text = element.get_text()

    # attempt iso8601 duration parsing
    if time_text.startswith("PT"):
        try:
            duration = isodate.parse_duration(time_text)
            return math.ceil(duration.total_seconds() / 60)
        except Exception:
            pass

    if time_text.startswith("P") and "T" in time_text:
        time_text = time_text.split("T", 2)[1]
    if "-" in time_text:
        time_text = time_text.split("-", 2)[
            1
        ]  # sometimes formats are like this: '12-15 minutes'
    if " to " in time_text:
        time_text = time_text.split("to", 2)[
            1
        ]  # sometimes formats are like this: '12 to 15 minutes'
    if "h" in time_text:
        time_text = time_text.replace("h", "hours")

    matched = TIME_REGEX.search(time_text)

    minutes = int(matched.groupdict().get("minutes") or 0)
    hours_matched = matched.groupdict().get("hours")
    days_matched = matched.groupdict().get("days")

    # workaround for formats like: 0D4H45M, that are not a valid iso8601 it seems
    if days_matched:
        minutes += 60 * 60 * float(days_matched.strip())
    if hours_matched:
        hours_matched = hours_matched.strip()
        if any([symbol in FRACTIONS.keys() for symbol in hours_matched]):
            hours = 0
            for fraction, value in FRACTIONS.items():
                if fraction in hours_matched:
                    hours += value
                    hours_matched = hours_matched.replace(fraction, "")
            hours += int(hours_matched) if hours_matched else 0
        elif "/" in hours_matched:
            # for example "1 1/2" is matched
            hours_matched_split = hours_matched.split(" ")
            hours = 0
            if len(hours_matched_split) == 2:
                hours += int(hours_matched_split[0])
            fraction = hours_matched_split[-1:][0].split("/")
            hours += float(int(fraction[0]) / int(fraction[1]))
        else:
            hours = float(hours_matched)

        minutes += round(60 * hours, 0)

    return minutes


def get_yields(element):
    """
    Will return a string of servings or items, if the receipt is for number of items and not servings
    the method will return the string "x item(s)" where x is the quantity.
    :param element: Should be BeautifulSoup.TAG, in some cases not feasible and will then be text.
    :return: The number of servings or items.
    """
    if element is None:
        raise ElementNotFoundInHtml(element)

    if isinstance(element, str):
        serve_text = element
    else:
        serve_text = element.get_text()

    if SERVE_REGEX_TO.search(serve_text):
        serve_text = serve_text.split(SERVE_REGEX_TO.split(serve_text, 2)[1], 2)[1]

    matched = SERVE_REGEX_NUMBER.search(serve_text).groupdict().get("items") or 0
    servings = "{} serving{}".format(matched, "" if int(matched) == 1 else "s")

    if SERVE_REGEX_ITEMS.search(serve_text) is not None:
        # This assumes if object(s), like sandwiches, it is 1 person.
        # Issue: "Makes one 9-inch pie, (realsimple-testcase, gives "9 items")
        servings = "{} item{}".format(matched, "" if int(matched) == 1 else "s")

    return servings


def normalize_string(string):
    # Convert all named and numeric character references (e.g. &gt;, &#62;)
    unescaped_string = html.unescape(string)
    return re.sub(
        r"\s+",
        " ",
        unescaped_string.replace("\xa0", " ")
        .replace("\n", " ")  # &nbsp;
        .replace("\t", " ")
        .strip(),
    )


def url_path_to_dict(path):
    pattern = (
        r"^"
        r"((?P<schema>.+?)://)?"
        r"((?P<user>.+?)(:(?P<password>.*?))?@)?"
        r"(?P<host>.*?)"
        r"(:(?P<port>\d+?))?"
        r"(?P<path>/.*?)?"
        r"(?P<query>[?].*?)?"
        r"$"
    )
    regex = re.compile(pattern)
    matches = regex.match(path)
    url_dict = matches.groupdict() if matches is not None else None

    return url_dict


def get_host_name(url):
    return url_path_to_dict(url.replace("://www.", "://"))["host"]


def change_keys(obj, convert):
    """
    Recursively goes through the dictionary obj and replaces keys with the convert function

    Useful for fixing incorrect property keys, e.g. in JSON-LD dictionaries

    Credit: StackOverflow user 'baldr'
    (https://web.archive.org/web/20201022163147/https://stackoverflow.com/questions/11700705/python-recursively-replace
        -character-in-keys-of-nested-dictionary/33668421)
    """
    if isinstance(obj, (str, int, float)):
        return obj
    if isinstance(obj, dict):
        new = obj.__class__()
        for k, v in obj.items():
            new[convert(k)] = change_keys(v, convert)
    elif isinstance(obj, (list, set, tuple)):
        new = obj.__class__(change_keys(v, convert) for v in obj)
    else:
        return obj
    return new



from recipe_scrapers.food import Food
from tests import ScraperTest


class TestFoodScraper(ScraperTest):

    scraper_class = Food

    def test_host(self):
        self.assertEqual("food.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.food.com/recipe/chicken-noodle-soup-with-carrots-parsnips-and-dill-454415",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Chicken Noodle Soup With Carrots, Parsnips and Dill",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "JackieOhNo")

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "8 cups low sodium chicken broth",
                "1 onion, chopped",
                "4 carrots, halved lengthwise and cut crosswise into 1-inch pieces",
                "4 parsnips, halved lengthwise and cut crosswise into 1-inch pieces",
                "1 1/2 teaspoons salt",
                "1/4 teaspoon fresh ground black pepper",
                "1 split chicken breast",
                "1 cup noodles (about 2 ounces)",
                "1/4 cup chopped fresh dill",
                "1/4 cup chopped fresh parsley",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "In a large pot, combine the broth, onion, carrots, parsnips, salt, and pepper and bring to a simmer. Add the chicken breasts to the pot and simmer until jfor about 20 minutes, until cooked. Remove the chicken and let rest. When cool enough to handle, remove skin and bones and chop or shred intobite-size pieces.\nWhile chicken is cooling, bring the soup back to a simmer and stir the noodles into the soup. Simmer until the vegetables are tender and the noodles are done, about 5 minutes. Return the chicken pieces to the pot and then stir in the dill and the parsley.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



from __future__ import annotations

import contextlib
from typing import Any, Optional

from ._abstract import AbstractScraper
from ._exceptions import NoSchemaFoundInWildMode, WebsiteNotImplementedError
from ._factory import SchemaScraperFactory
from ._utils import get_host_name
from .abril import Abril
from .acouplecooks import ACoupleCooks
from .afghankitchenrecipes import AfghanKitchenRecipes
from .akispetretzikis import AkisPetretzikis
from .albertheijn import AlbertHeijn
from .allrecipes import AllRecipes
from .alltomat import AllTomat
from .altonbrown import AltonBrown
from .amazingribs import AmazingRibs
from .ambitiouskitchen import AmbitiousKitchen
from .archanaskitchen import ArchanasKitchen
from .arla import Arla
from .atelierdeschefs import AtelierDesChefs
from .averiecooks import AverieCooks
from .bakingmischeif import BakingMischeif
from .bakingsense import BakingSense
from .bbcfood import BBCFood
from .bbcgoodfood import BBCGoodFood
from .bettybossi import BettyBossi
from .bettycrocker import BettyCrocker
from .biancazapatka import BiancaZapatka
from .bigoven import BigOven
from .blueapron import BlueApron
from .bodybuilding import Bodybuilding
from .bonappetit import BonAppetit
from .bowlofdelicious import BowlOfDelicious
from .briceletbaklava import BricelEtBaklava
from .budgetbytes import BudgetBytes
from .castironketo import CastIronKeto
from .cdkitchen import CdKitchen
from .chefkoch import Chefkoch
from .chefnini import Chefnini
from .closetcooking import ClosetCooking
from .comidinhasdochef import ComidinhasDoChef
from .cookeatshare import CookEatShare
from .cookieandkate import CookieAndKate
from .cookingcircle import CookingCircle
from .cookinglight import CookingLight
from .cookpad import CookPad
from .cookstr import Cookstr
from .coop import Coop
from .copykat import CopyKat
from .countryliving import CountryLiving
from .cucchiaio import Cucchiaio
from .cuisineaz import CuisineAZ
from .cybercook import Cybercook
from .davidlebovitz import DavidLebovitz
from .delish import Delish
from .ditchthecarbs import DitchTheCarbs
from .domesticateme import DomesticateMe
from .downshiftology import Downshiftology
from .dr import Dr
from .eatingbirdfood import EatingBirdFood
from .eatingwell import EatingWell
from .eatsmarter import Eatsmarter
from .eattolerant import EatTolerant
from .eatwhattonight import EatWhatTonight
from .emmikochteinfach import EmmiKochtEinfach
from .epicurious import Epicurious
from .ethanchlebowski import EthanChlebowski
from .farmhousedelivery import FarmhouseDelivery
from .fifteenspatulas import FifteenSpatulas
from .finedininglovers import FineDiningLovers
from .fitmencook import FitMenCook
from .food import Food
from .food52 import Food52
from .foodandwine import FoodAndWine
from .foodnetwork import FoodNetwork
from .foodrepublic import FoodRepublic
from .forksoverknives import ForksOverKnives
from .forktospoon import ForkToSpoon
from .franzoesischkochen import FranzoesischKochen
from .fredriksfikaallas import FredriksFikaAllas
from .g750g import G750g
from .geniuskitchen import GeniusKitchen
from .giallozafferano import GialloZafferano
from .gimmesomeoven import GimmeSomeOven
from .globo import Globo
from .gonnawantseconds import GonnaWantSeconds
from .goustojson import GoustoJson
from .greatbritishchefs import GreatBritishChefs
from .halfbakedharvest import HalfBakedHarvest
from .handletheheat import HandleTheHeat
from .hassanchef import HassanChef
from .headbangerskitchen import HeadbangersKitchen
from .heb import HEB
from .heinzbrasil import HeinzBrasil
from .hellofresh import HelloFresh
from .homechef import HomeChef
from .hostthetoast import Hostthetoast
from .hundredandonecookbooks import HundredAndOneCookbooks
from .ica import Ica
from .ig import IG
from .imworthy import ImWorthy
from .indianhealthyrecipes import IndianHealthyRecipes
from .innit import Innit
from .inspiralized import Inspiralized
from .jamieoliver import JamieOliver
from .jimcooksfoodgood import JimCooksFoodGood
from .joyfoodsunshine import Joyfoodsunshine
from .justataste import JustATaste
from .justbento import JustBento
from .justonecookbook import JustOneCookbook
from .kennymcgovern import KennyMcGovern
from .kingarthur import KingArthur
from .kitchenstories import KitchenStories
from .kochbar import Kochbar
from .koket import Koket
from .kptncook import KptnCook
from .kuchniadomowa import KuchniaDomowa
from .kwestiasmaku import KwestiaSmaku
from .latelierderoxane import LAtelierDeRoxane
from .leanandgreenrecipes import LeanAndGreenRecipes
from .lecker import Lecker
from .lecremedelacrumb import LeCremeDeLaCrumb
from .lekkerensimpel import LekkerEnSimpel
from .littlespicejar import LittleSpiceJar
from .livelytable import LivelyTable
from .lovingitvegan import Lovingitvegan
from .maangchi import Maangchi
from .madensverden import MadensVerden
from .madewithlau import MadeWithLau
from .marleyspoon import MarleySpoon
from .marmiton import Marmiton
from .marthastewart import MarthaStewart
from .matprat import Matprat
from .meljoulwan import Meljoulwan
from .melskitchencafe import MelsKitchenCafe
from .mindmegette import Mindmegette
from .minimalistbaker import Minimalistbaker
from .misya import Misya
from .mobkitchen import MobKitchen
from .momswithcrockpots import MomsWithCrockPots
from .monsieurcuisine import MonsieurCuisine
from .motherthyme import MotherThyme
from .mybakingaddiction import MyBakingAddiction
from .mykitchen101 import MyKitchen101
from .mykitchen101en import MyKitchen101en
from .myrecipes import MyRecipes
from .nhshealthierfamilies import NHSHealthierFamilies
from .nihhealthyeating import NIHHealthyEating
from .nourishedbynutrition import NourishedByNutrition
from .nutritionbynathalie import NutritionByNathalie
from .nytimes import NYTimes
from .ohsheglows import OhSheGlows
from .omnivorescookbook import OmnivoresCookbook
from .onehundredonecookbooks import OneHundredOneCookBooks
from .owenhan import OwenHan
from .paleorunningmomma import PaleoRunningMomma
from .panelinha import Panelinha
from .paninihappy import PaniniHappy
from .pingodoce import PingoDoce
from .popsugar import PopSugar
from .practicalselfreliance import PracticalSelfReliance
from .pressureluckcooking import PressureLuckCooking
from .primaledgehealth import PrimalEdgeHealth
from .projectgezond import ProjectGezond
from .przepisy import Przepisy
from .purelypope import PurelyPope
from .purplecarrot import PurpleCarrot
from .rachlmansfield import RachlMansfield
from .rainbowplantlife import RainbowPlantLife
from .realfoodtesco import RealFoodTesco
from .realsimple import RealSimple
from .recipetineats import RecipeTinEats
from .redhousespice import RedHouseSpice
from .reishunger import Reishunger
from .rezeptwelt import Rezeptwelt
from .rosannapansino import RosannaPansino
from .rutgerbakt import RutgerBakt
from .sallysbakingaddiction import SallysBakingAddiction
from .saveur import Saveur
from .seriouseats import SeriousEats
from .simpleveganista import SimpleVeganista
from .simplycookit import SimplyCookit
from .simplyquinoa import SimplyQuinoa
from .simplyrecipes import SimplyRecipes
from .simplywhisked import SimplyWhisked
from .skinnytaste import SkinnyTaste
from .smulweb import Smulweb
from .southerncastiron import SouthernCastIron
from .southernliving import SouthernLiving
from .spendwithpennies import SpendWithPennies
from .springlane import Springlane
from .steamykitchen import SteamyKitchen
from .streetkitchen import StreetKitchen
from .sunbasket import SunBasket
from .sundpaabudget import SundPaaBudget
from .sunset import Sunset
from .sweetcsdesigns import SweetCsDesigns
from .sweetpeasandsaffron import SweetPeasAndSaffron
from .tasteofhome import TasteOfHome
from .tastesbetterfromscratch import TastesBetterFromScratch
from .tastesoflizzyt import TastesOfLizzyT
from .tasty import Tasty
from .tastykitchen import TastyKitchen
from .theclevercarrot import TheCleverCarrot
from .thehappyfoodie import TheHappyFoodie
from .thekitchenmagpie import TheKitchenMagPie
from .thekitchn import TheKitchn
from .thenutritiouskitchen import TheNutritiousKitchen
from .thepioneerwoman import ThePioneerWoman
from .thespruceeats import TheSpruceEats
from .thevintagemixer import TheVintageMixer
from .thewoksoflife import Thewoksoflife
from .timesofindia import TimesOfIndia
from .tineno import TineNo
from .tudogostoso import TudoGostoso
from .twopeasandtheirpod import TwoPeasAndTheirPod
from .usdamyplate import USDAMyPlate
from .valdemarsro import Valdemarsro
from .vanillaandbean import VanillaAndBean
from .vegolosi import Vegolosi
from .vegrecipesofindia import VegRecipesOfIndia
from .watchwhatueat import WatchWhatUEat
from .weightwatchers import WeightWatchers
from .weightwatcherspublic import WeightWatchersPublic
from .whatsgabycooking import WhatsGabyCooking
from .wholefoods import WholeFoods
from .wikicookbook import WikiCookbook
from .woolworths import Woolworths
from .woop import Woop
from .yemek import Yemek
from .yummly import Yummly
from .zeitwochenmarkt import ZeitWochenmarkt
from .zenbelly import ZenBelly

SCRAPERS = {
    ACoupleCooks.host(): ACoupleCooks,
    Abril.host(): Abril,
    AfghanKitchenRecipes.host(): AfghanKitchenRecipes,
    AkisPetretzikis.host(): AkisPetretzikis,
    AlbertHeijn.host(): AlbertHeijn,
    AllRecipes.host(): AllRecipes,
    AllTomat.host(): AllTomat,
    AltonBrown.host(): AltonBrown,
    AmazingRibs.host(): AmazingRibs,
    AmbitiousKitchen.host(): AmbitiousKitchen,
    ArchanasKitchen.host(): ArchanasKitchen,
    Arla.host(): Arla,
    AtelierDesChefs.host(): AtelierDesChefs,
    AverieCooks.host(): AverieCooks,
    BBCFood.host(): BBCFood,
    BBCFood.host(domain="co.uk"): BBCFood,
    BBCGoodFood.host(): BBCGoodFood,
    BakingSense.host(): BakingSense,
    BakingMischeif.host(): BakingMischeif,
    BettyBossi.host(): BettyBossi,
    BettyCrocker.host(): BettyCrocker,
    BiancaZapatka.host(): BiancaZapatka,
    BigOven.host(): BigOven,
    BlueApron.host(): BlueApron,
    Bodybuilding.host(): Bodybuilding,
    BonAppetit.host(): BonAppetit,
    BowlOfDelicious.host(): BowlOfDelicious,
    BricelEtBaklava.host(): BricelEtBaklava,
    BudgetBytes.host(): BudgetBytes,
    CastIronKeto.host(): CastIronKeto,
    CdKitchen.host(): CdKitchen,
    Chefkoch.host(): Chefkoch,
    Chefnini.host(): Chefnini,
    ClosetCooking.host(): ClosetCooking,
    ComidinhasDoChef.host(): ComidinhasDoChef,
    CookEatShare.host(): CookEatShare,
    CookPad.host(): CookPad,
    CookieAndKate.host(): CookieAndKate,
    CookingCircle.host(): CookingCircle,
    CookingLight.host(): CookingLight,
    Cookstr.host(): Cookstr,
    Coop.host(): Coop,
    CopyKat.host(): CopyKat,
    CountryLiving.host(): CountryLiving,
    Cucchiaio.host(): Cucchiaio,
    CuisineAZ.host(): CuisineAZ,
    Cybercook.host(): Cybercook,
    DavidLebovitz.host(): DavidLebovitz,
    Delish.host(): Delish,
    DitchTheCarbs.host(): DitchTheCarbs,
    DomesticateMe.host(): DomesticateMe,
    Downshiftology.host(): Downshiftology,
    Dr.host(): Dr,
    EatWhatTonight.host(): EatWhatTonight,
    EatingBirdFood.host(): EatingBirdFood,
    EatingWell.host(): EatingWell,
    Eatsmarter.host(): Eatsmarter,
    Eatsmarter.host(domain="de"): Eatsmarter,
    EatTolerant.host(): EatTolerant,
    EmmiKochtEinfach.host(): EmmiKochtEinfach,
    Epicurious.host(): Epicurious,
    EthanChlebowski.host(): EthanChlebowski,
    FarmhouseDelivery.host(): FarmhouseDelivery,
    FifteenSpatulas.host(): FifteenSpatulas,
    FineDiningLovers.host(): FineDiningLovers,
    FitMenCook.host(): FitMenCook,
    Food.host(): Food,
    Food52.host(): Food52,
    FoodAndWine.host(): FoodAndWine,
    FoodNetwork.host(): FoodNetwork,
    FoodRepublic.host(): FoodRepublic,
    ForkToSpoon.host(): ForkToSpoon,
    ForksOverKnives.host(): ForksOverKnives,
    FranzoesischKochen.host(): FranzoesischKochen,
    FredriksFikaAllas.host(): FredriksFikaAllas,
    G750g.host(): G750g,
    GeniusKitchen.host(): GeniusKitchen,
    GialloZafferano.host(): GialloZafferano,
    GimmeSomeOven.host(): GimmeSomeOven,
    Globo.host(): Globo,
    GonnaWantSeconds.host(): GonnaWantSeconds,
    GoustoJson.host(): GoustoJson,
    GreatBritishChefs.host(): GreatBritishChefs,
    HEB.host(): HEB,
    HalfBakedHarvest.host(): HalfBakedHarvest,
    HandleTheHeat.host(): HandleTheHeat,
    HassanChef.host(): HassanChef,
    HeadbangersKitchen.host(): HeadbangersKitchen,
    HeinzBrasil.host(): HeinzBrasil,
    HelloFresh.host(): HelloFresh,
    HelloFresh.host(domain="co.uk"): HelloFresh,
    HelloFresh.host(domain="de"): HelloFresh,
    HelloFresh.host(domain="fr"): HelloFresh,
    HelloFresh.host(domain="nl"): HelloFresh,
    HelloFresh.host(domain="ie"): HelloFresh,
    HomeChef.host(): HomeChef,
    Hostthetoast.host(): Hostthetoast,
    HundredAndOneCookbooks.host(): HundredAndOneCookbooks,
    Ica.host(): Ica,
    ImWorthy.host(): ImWorthy,
    IG.host(): IG,
    IndianHealthyRecipes.host(): IndianHealthyRecipes,
    Innit.host(): Innit,
    Inspiralized.host(): Inspiralized,
    JamieOliver.host(): JamieOliver,
    JimCooksFoodGood.host(): JimCooksFoodGood,
    Joyfoodsunshine.host(): Joyfoodsunshine,
    JustATaste.host(): JustATaste,
    JustBento.host(): JustBento,
    JustOneCookbook.host(): JustOneCookbook,
    KennyMcGovern.host(): KennyMcGovern,
    KingArthur.host(): KingArthur,
    KitchenStories.host(): KitchenStories,
    Kochbar.host(): Kochbar,
    Koket.host(): Koket,
    KptnCook.host(): KptnCook,
    KptnCook.host(subdomain="sharing"): KptnCook,
    KuchniaDomowa.host(): KuchniaDomowa,
    KwestiaSmaku.host(): KwestiaSmaku,
    LAtelierDeRoxane.host(): LAtelierDeRoxane,
    LeCremeDeLaCrumb.host(): LeCremeDeLaCrumb,
    LeanAndGreenRecipes.host(): LeanAndGreenRecipes,
    Lecker.host(): Lecker,
    LekkerEnSimpel.host(): LekkerEnSimpel,
    LittleSpiceJar.host(): LittleSpiceJar,
    LivelyTable.host(): LivelyTable,
    Lovingitvegan.host(): Lovingitvegan,
    Maangchi.host(): Maangchi,
    MadensVerden.host(): MadensVerden,
    MadeWithLau.host(): MadeWithLau,
    MarleySpoon.host(): MarleySpoon,
    MarleySpoon.host(domain="de"): MarleySpoon,
    MarleySpoon.host(domain="com.au"): MarleySpoon,
    MarleySpoon.host(domain="be"): MarleySpoon,
    MarleySpoon.host(domain="nl"): MarleySpoon,
    MarleySpoon.host(domain="at"): MarleySpoon,
    MarleySpoon.host(domain="se"): MarleySpoon,
    Marmiton.host(): Marmiton,
    MarthaStewart.host(): MarthaStewart,
    Matprat.host(): Matprat,
    Meljoulwan.host(): Meljoulwan,
    MelsKitchenCafe.host(): MelsKitchenCafe,
    Mindmegette.host(): Mindmegette,
    Minimalistbaker.host(): Minimalistbaker,
    Misya.host(): Misya,
    MobKitchen.host(domain="mob.co.uk"): MobKitchen,
    MobKitchen.host(domain="mobkitchen.co.uk"): MobKitchen,
    MomsWithCrockPots.host(): MomsWithCrockPots,
    MonsieurCuisine.host(): MonsieurCuisine,
    MotherThyme.host(): MotherThyme,
    MyBakingAddiction.host(): MyBakingAddiction,
    MyKitchen101.host(): MyKitchen101,
    MyKitchen101en.host(): MyKitchen101en,
    MyRecipes.host(): MyRecipes,
    NHSHealthierFamilies.host(): NHSHealthierFamilies,
    NIHHealthyEating.host(): NIHHealthyEating,
    NYTimes.host(): NYTimes,
    NourishedByNutrition.host(): NourishedByNutrition,
    NutritionByNathalie.host(): NutritionByNathalie,
    OhSheGlows.host(): OhSheGlows,
    OmnivoresCookbook.host(): OmnivoresCookbook,
    OneHundredOneCookBooks.host(): OneHundredOneCookBooks,
    OwenHan.host(): OwenHan,
    PaleoRunningMomma.host(): PaleoRunningMomma,
    Panelinha.host(): Panelinha,
    PaniniHappy.host(): PaniniHappy,
    PingoDoce.host(): PingoDoce,
    PopSugar.host(): PopSugar,
    PracticalSelfReliance.host(): PracticalSelfReliance,
    PracticalSelfReliance.host(domain="creativecanning.com"): PracticalSelfReliance,
    PressureLuckCooking.host(): PressureLuckCooking,
    PrimalEdgeHealth.host(): PrimalEdgeHealth,
    ProjectGezond.host(): ProjectGezond,
    Przepisy.host(): Przepisy,
    PurelyPope.host(): PurelyPope,
    PurpleCarrot.host(): PurpleCarrot,
    RachlMansfield.host(): RachlMansfield,
    RainbowPlantLife.host(): RainbowPlantLife,
    RealFoodTesco.host(): RealFoodTesco,
    RealSimple.host(): RealSimple,
    RealFoodTesco.host(): RealFoodTesco,
    RecipeTinEats.host(): RecipeTinEats,
    RedHouseSpice.host(): RedHouseSpice,
    Reishunger.host(): Reishunger,
    Rezeptwelt.host(): Rezeptwelt,
    RosannaPansino.host(): RosannaPansino,
    RutgerBakt.host(): RutgerBakt,
    SallysBakingAddiction.host(): SallysBakingAddiction,
    Saveur.host(): Saveur,
    SeriousEats.host(): SeriousEats,
    SimpleVeganista.host(): SimpleVeganista,
    SimplyCookit.host(): SimplyCookit,
    SimplyQuinoa.host(): SimplyQuinoa,
    SimplyRecipes.host(): SimplyRecipes,
    SimplyWhisked.host(): SimplyWhisked,
    SkinnyTaste.host(): SkinnyTaste,
    Smulweb.host(): Smulweb,
    SouthernCastIron.host(): SouthernCastIron,
    SouthernLiving.host(): SouthernLiving,
    SpendWithPennies.host(): SpendWithPennies,
    Springlane.host(): Springlane,
    SteamyKitchen.host(): SteamyKitchen,
    StreetKitchen.host(): StreetKitchen,
    SunBasket.host(): SunBasket,
    SundPaaBudget.host(): SundPaaBudget,
    Sunset.host(): Sunset,
    SweetCsDesigns.host(): SweetCsDesigns,
    SweetPeasAndSaffron.host(): SweetPeasAndSaffron,
    TasteOfHome.host(): TasteOfHome,
    TastesBetterFromScratch.host(): TastesBetterFromScratch,
    TastesOfLizzyT.host(): TastesOfLizzyT,
    Tasty.host(): Tasty,
    TastyKitchen.host(): TastyKitchen,
    TheCleverCarrot.host(): TheCleverCarrot,
    TheHappyFoodie.host(): TheHappyFoodie,
    TheKitchenMagPie.host(): TheKitchenMagPie,
    TheKitchn.host(): TheKitchn,
    TheNutritiousKitchen.host(): TheNutritiousKitchen,
    ThePioneerWoman.host(): ThePioneerWoman,
    TheSpruceEats.host(): TheSpruceEats,
    TheVintageMixer.host(): TheVintageMixer,
    Thewoksoflife.host(): Thewoksoflife,
    TimesOfIndia.host(): TimesOfIndia,
    TineNo.host(): TineNo,
    TudoGostoso.host(): TudoGostoso,
    TwoPeasAndTheirPod.host(): TwoPeasAndTheirPod,
    USDAMyPlate.host(): USDAMyPlate,
    Valdemarsro.host(): Valdemarsro,
    VanillaAndBean.host(): VanillaAndBean,
    VegRecipesOfIndia.host(): VegRecipesOfIndia,
    Vegolosi.host(): Vegolosi,
    WatchWhatUEat.host(): WatchWhatUEat,
    WeightWatchers.host(): WeightWatchers,
    WeightWatchersPublic.host(): WeightWatchersPublic,
    WhatsGabyCooking.host(): WhatsGabyCooking,
    WholeFoods.host(): WholeFoods,
    WholeFoods.host(domain="co.uk"): WholeFoods,
    Woop.host(): Woop,
    WikiCookbook.host(): WikiCookbook,
    Woolworths.host(): Woolworths,
    Yemek.host(): Yemek,
    Yummly.host(): Yummly,
    ZeitWochenmarkt.host(): ZeitWochenmarkt,
    ZenBelly.host(): ZenBelly,
}


def get_supported_urls() -> set[str]:
    return set(SCRAPERS.keys())


def scraper_exists_for(url_path: str) -> bool:
    host_name = get_host_name(url_path)
    return host_name in get_supported_urls()


def scrape_me(url_path: str, **options: Any) -> AbstractScraper:
    host_name = get_host_name(url_path)

    try:
        scraper = SCRAPERS[host_name]
    except KeyError:
        if not options.get("wild_mode", False):
            raise WebsiteNotImplementedError(host_name)
        else:
            options.pop("wild_mode")
            wild_scraper = SchemaScraperFactory.generate(url_path, **options)
            if not wild_scraper.schema.data:
                raise NoSchemaFoundInWildMode(url_path)
            return wild_scraper

    return scraper(url_path, **options)


def scrape_html(
    html: str, org_url: Optional[str] = None, **options: dict[str, Any]
) -> AbstractScraper:
    """
    takes a string of html and returns a scraper object. if the org_url is specified
    then the scraper will use that url to resolve a defined scraper, otherwise it will
    fall back to wild mode. If no schema is found in wild mode then a
    NoSchemaFoundInWildMode exception will be raised.

    Args:
        html (str): raw HTML in text form
        org_url (Optional[str], optional): Original URL of the HTML. Defaults to None.

    Raises:
        NoSchemaFoundInWildMode: If no schema is found in wild mode.

    Returns:
        AbstractScraper:
    """

    host_name = get_host_name(org_url) if org_url is not None else None

    scraper = None
    if host_name:
        with contextlib.suppress(KeyError):
            scraper = SCRAPERS[host_name]

    if not scraper:
        wild_scraper = SchemaScraperFactory.generate(url=org_url, html=html, **options)

        if not wild_scraper.schema.data:
            raise NoSchemaFoundInWildMode(org_url)

        return wild_scraper

    return scraper(url=org_url, html=html, **options)


__all__ = ["scrape_me", "scrape_html"]
name = "recipe_scrapers"



from recipe_scrapers.paninihappy import PaniniHappy
from tests import ScraperTest


class TestPaniniHappyScraper(ScraperTest):

    scraper_class = PaniniHappy

    def test_host(self):
        self.assertEqual("paninihappy.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "http://paninihappy.com/grilled-mac-cheese-with-bbq-pulled-pork/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Grilled Mac & Cheese with BBQ Pulled Pork"
        )

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 items", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "paninihappy_files/Grilled_Mac_and_Cheese-main-490.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "4 tablespoons unsalted butter, divided",
                "4 cups prepared macaroni and cheese, warmed",
                "2 onions, thinly sliced",
                "Kosher salt and freshly ground pepper",
                "1 cup barbecue sauce",
                "2 cups prepared pulled pork",
                "8 slices sourdough bread",
                "12 slices sharp cheddar cheese (about 6 ounces)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Spread the macaroni and cheese in an 8-inch-square baking dish to about 3/4 inch thick. Cover with plastic wrap and chill until firm, about 45 minutes. Cut the macaroni and cheese into squares that are slightly smaller than the bread slices.\nMeanwhile, melt 2 tablespoons butter in a skillet over medium heat. Add the onions and cook, stirring, until caramelized, about 20 minutes. Season with salt and pepper.\nCombine the barbecue sauce and pulled pork in a saucepan over low heat and cook until warmed through, about 5 minutes.\nPreheat the panini grill to medium-high heat.\nMelt the remaining 2 tablespoons butter and brush on one side of each bread slice. Flip over half of the bread slices; layer 1 slice of cheddar, 1 macaroni-and-cheese square and another slice of cheddar on each. Top each with one-quarter of the pulled pork and caramelized onions and another slice of cheddar. Top with the remaining bread slices, buttered-side up.\nWorking in batches, cook the sandwiches until the cheese melts and the bread is golden, about 5 minutes.",
            self.harvester_class.instructions(),
        )
